package im.zego.minigameenginedemo;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.IBinder;
import android.util.Log;

import androidx.annotation.Nullable;

/**
 * 用来保活
 */
public class ForegroundService extends Service {
    public ForegroundService() {
    }
    private static final String TAG = "ForegroundService"; // 定义一个常量TAG，用于日志输出
    private static final int SERVICE_ID = 1; // 定义一个常量SERVICE_ID，用于前台服务的ID

    @Override
    public void onCreate() {
        Log.i(TAG, "[onCreate]"); // 打印日志，表示服务正在创建
        createNotificationChannel(); // 调用创建通知渠道的方法
    }

    @Override
    public void onDestroy() {
        Log.i(TAG, "[onDestroy]"); // 打印日志，表示服务正在销毁
        stopForeground(true); // 停止前台服务
        super.onDestroy();
    }

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    private void createNotificationChannel() {
        Notification.Builder builder = new Notification.Builder(this.getApplicationContext()); // 获取一个Notification构造器
        builder.setWhen(System.currentTimeMillis()) // 设置该通知发生的时间
                .setOngoing(true) // 设置通知为持续显示
                .setVisibility(Notification.VISIBILITY_PUBLIC); // 设置通知的可见性为公开

        /*以下是对Android 8.0的适配*/
        //普通notification适配
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) { // 判断当前Android版本是否大于等于Android 8.0
            builder.setChannelId("notification_id"); // 设置通知的渠道ID
            NotificationManager notificationManager = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE); // 获取NotificationManager实例
            NotificationChannel channel = new NotificationChannel("notification_id", "notification_name", NotificationManager.IMPORTANCE_DEFAULT); // 创建通知渠道
            notificationManager.createNotificationChannel(channel); // 将通知渠道添加到NotificationManager
        }
        Notification notification = builder.build(); // 获取构建好的Notification
        startForeground(SERVICE_ID, notification); // 启动前台服务，传入SERVICE_ID和Notification
    }
}