package im.zego.minigameenginedemo.audio;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.media.AudioManager;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;

import java.util.HashMap;

import im.zego.minigameenginedemo.IExecuteCallback;
import im.zego.minigameenginedemo.RTCEngineManager;
import im.zego.minigameenginedemo.utils.CommonUtils;

public class AudioPlayer {
    private final String VOLUME_CHANGE_ACTION = "android.media.VOLUME_CHANGED_ACTION";
    private HashMap<String, Integer> nameSoundIDMap = new HashMap<>();
    private HashMap<String, String> namePathMap = new HashMap<>();
    private String savePath = CommonUtils.getDataBaseFile(CommonUtils.getApplication()) + "/audio/";
    //实例化AudioManager对象，控制声音
    private AudioManager am;
    //最大音量
    private float audioMaxVolume;
    //当前音量
    private float audioCurrentVolume;
    private float volumeRatio;
    private RTCEngineManager rtcEngineManager = RTCEngineManager.getInstance();

    public AudioPlayer() {
        //实例化AudioManager对象，控制声音
//        am = (AudioManager)CommonUtils.getApplication().getSystemService(Context.AUDIO_SERVICE);
//        //最大音量
//        audioMaxVolume = am.getStreamMaxVolume(AudioManager.STREAM_MUSIC);
//        //当前音量
//        audioCurrentVolume = am.getStreamVolume(AudioManager.STREAM_MUSIC);
        volumeRatio = 1f;//audioCurrentVolume / audioMaxVolume;
//        IntentFilter filter = new IntentFilter();
//        filter.addAction(VOLUME_CHANGE_ACTION);
//        CommonUtils.getApplication().registerReceiver(volumeChangReceiver, filter);
    }

//    private BroadcastReceiver volumeChangReceiver = new BroadcastReceiver() {
//        @Override
//        public void onReceive(Context context, Intent intent) {
//            if (VOLUME_CHANGE_ACTION.equals(intent.getAction()) &&
//                    intent.getIntExtra("android.media.EXTRA_VOLUME_STREAM_TYPE", -1) == AudioManager.STREAM_MUSIC) {
//                audioCurrentVolume = am.getStreamVolume(AudioManager.STREAM_MUSIC);
//                volumeRatio = 1f;//audioCurrentVolume / audioMaxVolume;
//            }
//        }
//    };

    public void playSound(String name, boolean isPlay, String url, boolean isLoop, int volume) {
        String path = namePathMap.get(url);
        if (path == null) {
            if (url.startsWith("file://")) {
                namePathMap.put(url, url);
                String filePath = url;
                Integer soundID = nameSoundIDMap.get(name);
                if (isPlay) {
                    if (soundID == null) {
                        rtcEngineManager.loadSoundEffect(filePath, (errorCode1, soundEffectID) -> {
                            final int newSoundID = soundEffectID.intValue();
                            nameSoundIDMap.put(name, newSoundID);
                            rtcEngineManager.startSound(newSoundID, filePath, isLoop, volumeRatio * volume / 100);
                        });
                    } else {
                        rtcEngineManager.startSound(soundID, filePath, isLoop, volumeRatio * volume / 100);
                    }
                } else {
                    if (soundID != null) {
                        rtcEngineManager.stopSound(soundID);
                    }
                }
            } else {
                // 下载
                CommonUtils.downloadFile(url, savePath, new IExecuteCallback<String>() {
                    @Override
                    public void onResult(int errorCode, String object) {
                        if (errorCode == 0 && !TextUtils.isEmpty(object)) {
                            namePathMap.put(url, object);
                        }
                        String path = object;
                        Integer soundID = nameSoundIDMap.get(name);
                        if (isPlay) {
                            if (soundID == null) {
                                rtcEngineManager.loadSoundEffect(path, (errorCode1, soundEffectID) -> {
                                    final int newSoundID = soundEffectID.intValue();
                                    nameSoundIDMap.put(name, newSoundID);
                                    rtcEngineManager.startSound(newSoundID, path, isLoop, volumeRatio * volume / 100);
                                });

                                return;
                            } else {
                                rtcEngineManager.startSound(soundID, path, isLoop, volumeRatio * volume / 100);
                            }
                        } else {
                            if (soundID != null) {
                                rtcEngineManager.stopSound(soundID);
                            }
                        }
                    }
                });
            }
        } else {
            Integer soundID = nameSoundIDMap.get(name);
            if (isPlay) {
                if (soundID == null) {
                    rtcEngineManager.loadSoundEffect(path, (errorCode1, soundEffectID) -> {
                        final int newSoundID = soundEffectID.intValue();
                        nameSoundIDMap.put(name, newSoundID);
                        rtcEngineManager.startSound(newSoundID, path, isLoop, volumeRatio * volume / 100);
                    });

                    return;
                } else {
                    rtcEngineManager.startSound(soundID, path, isLoop, volumeRatio * volume / 100);
                }
            } else {
                if (soundID != null) {
                    rtcEngineManager.stopSound(soundID);
                }
            }
        }
    }

    public void changeSoundVolume(String name, int volume) {
        Integer soundID = nameSoundIDMap.get(name);
        if (soundID == null) {
            return;
        }

        rtcEngineManager.changeSoundVolume(soundID, volumeRatio * volume / 100);
    }

    public void release() {
        rtcEngineManager.destroyZegoAudioEffectPlayer();
//        CommonUtils.getApplication().unregisterReceiver(volumeChangReceiver);
    }

    public void pause() {
        rtcEngineManager.pauseSoundEffect();
    }

    public void resume() {
        rtcEngineManager.resumeSoundEffect();
    }
}
