package im.zego.minigameenginedemo.backend;

import static im.zego.minigameenginedemo.backend.BackendService.gson;

import com.google.gson.annotations.SerializedName;

public class BackendModel {

    // 服务器响应模型
    public static class ServerResponse {
        @SerializedName("Code")
        public int code; // 响应码
        @SerializedName("Message")
        public String message; // 响应消息
        @SerializedName("RequestId")
        public String requestID; // 请求ID
    }

    // 获取令牌请求模型
    public static class GetTokenRequest {
        @SerializedName("app_id")
        public long appID; // 应用ID

        @SerializedName("user_id")
        public String userID; // 用户ID

        // 构造函数
        public GetTokenRequest(long appID, String userID) {
            this.appID = appID;
            this.userID = userID;
        }

        // 将对象转换为JSON字符串
        @Override
        public String toString() {
            return gson.toJson(this);
        }
    }

    // 获取云渲染游戏运行Code请求
    public static class GetGameLaunchCodeRequest {
        /**
         * {
         *     "RoomId": "test123",
         *     "MiniGameId": "TinyLoveWar",
         *     "AnchorId": "zhubo11111",
         *     "Nickname": "主播",
         *     "Avatar": "www.",
         *     "Sex": 1
         * }
         */
        @SerializedName("AppId")
        public long appID; // 应用ID

        @SerializedName("RoomId")
        public String roomID; // 房间ID

        @SerializedName("MiniGameId")
        public String gameID; // 游戏ID

        @SerializedName("AnchorId")
        public String userID; // 用户ID

        @SerializedName("Avatar")
        public String avatar; // 用户头像

        @SerializedName("Sex")
        public int sex = 1; // 用户性别 1-男 2-女

        // 构造函数
        public GetGameLaunchCodeRequest(long appID, String userID, String roomID, String gameID, String avatar, int sex) {
            this.appID = appID;
            this.userID = userID;
            this.roomID = roomID;
            this.gameID = gameID;
            this.avatar = avatar;
            this.sex = sex;
        }

        // 将对象转换为JSON字符串
        @Override
        public String toString() {
            return gson.toJson(this);
        }
    }

    // 获取云渲染游戏运行Code回包
    public static class GetGameLaunchCodeResponse extends ServerResponse {
        @SerializedName("Data")
        public GetGameLaunchCodeData data; // 数据

        public static class GetGameLaunchCodeData {
            @SerializedName("GameLaunchCode")
            public String gameLaunchCode; // 游戏运行Code
        }
    }
}
