package im.zego.minigameenginedemo.backend;

import static im.zego.minigameenginedemo.KeyCenter.domain;

import static im.zego.minigameenginedemo.KeyCenter.domain;

import android.util.Log;

import androidx.annotation.NonNull;

import com.google.gson.Gson;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import im.zego.minigameenginedemo.IExecuteCallback;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

/**
 * 接入者提醒：从安全性上考虑，getToken, getLaunchCode 需要改成从接入方的后台来获取，下面的实现是从Demo的模拟后台获取的。
 */
public class BackendService {
    private static final String TAG = "BackendService";

    public static Gson gson = new Gson();

    // 获取 Token 的方法
    public static void getToken(long appID, String userID, IExecuteCallback<String> callback) {
        BackendModel.GetTokenRequest request = new BackendModel.GetTokenRequest(appID, userID);
        try {
            httpPost(domain + "/api/token", request.toString(), new IExecuteCallback<String>() {
                @Override
                public void onResult(int errorCode, String response) {
                    if (errorCode == 200) {
                        callback.onResult(0, response);
                    } else {
                        callback.onResult(errorCode, response);
                    }
                }
            });
        } catch (Exception e) {
            Log.e(TAG, "getToken exception:" + e.getMessage());
        }
    }

    public static void getLaunchCode(String userID, long appID, String gameID, String roomID, String avatar, int sex, IExecuteCallback<String> callback) {
        try {
            BackendModel.GetGameLaunchCodeRequest request = new BackendModel.GetGameLaunchCodeRequest(appID, userID, roomID, gameID, avatar, sex);
            httpPost(domain + "/api/getGameLaunchCode", request.toString(), new IExecuteCallback<String>() {
                @Override
                public void onResult(int errorCode, String response) {
                    if (errorCode == 200) {
                        BackendModel.GetGameLaunchCodeResponse getGameLaunchCodeResponse = gson.fromJson(response, BackendModel.GetGameLaunchCodeResponse.class);
                        String gameLaunchCode = "";
                        int retCode = getGameLaunchCodeResponse == null ? -1 : getGameLaunchCodeResponse.code;
                        if (getGameLaunchCodeResponse != null && getGameLaunchCodeResponse.code == 0 && getGameLaunchCodeResponse.data != null) {
                            gameLaunchCode = getGameLaunchCodeResponse.data.gameLaunchCode;
                            Log.i(TAG, "gameLaunchCode: " + getGameLaunchCodeResponse.data.gameLaunchCode);
                        }

                        callback.onResult(retCode, gameLaunchCode);
                    } else {
                        callback.onResult(errorCode, "");
                    }

                }
            });
        } catch (Exception e) {

        }
    }

    public static void httpPost(String strUrl, String body, IExecuteCallback<String> callback) throws MalformedURLException {
        OkHttpClient client = new OkHttpClient.Builder().build();
        URL url = new URL(strUrl);
        RequestBody requestBody = RequestBody.create(body, MediaType.parse("application/json;charset=UTF-8"));
        Request request = new Request.Builder().url(url) //
//                .addHeader("Content-Type", "application/json") //
//                .addHeader("Accept", "application/json,version=1.0") //
//                .addHeader("Date", date) //
                .addHeader("Host", url.getHost()) //
//                .addHeader("Digest", digest) //
                // 签名信息
//                .addHeader("Authorization", authorization) //
                .post(requestBody) //
                .build();
        Log.i(TAG, "[httpPost] " + request.toString() + ", body:" + body);
        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(@NonNull Call call, @NonNull IOException e) {
                Log.e(TAG, "[httpPost][onFailure] " + e.getMessage());
                callback.onResult(-1, "");
            }

            @Override
            public void onResponse(@NonNull Call call, @NonNull Response response) throws IOException {
                int code = response.code();
                String result = response.body().string();
                Log.i(TAG, "[httpPost][onResponse] " + result);
                response.close();
                callback.onResult(code, result);
            }
        });
    }
}
