package im.zego.minigameenginedemo.ui.main;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;
import java.util.List;

import im.zego.minigameengine.ZegoGameComponent;
import im.zego.minigameengine.ZegoGameInfoDetail;
import im.zego.minigameengine.ZegoGameLoadState;
import im.zego.minigameengine.ZegoGameMode;
import im.zego.minigameengine.ZegoGameState;
import im.zego.minigameengine.ZegoGameTaxType;
import im.zego.minigameengine.ZegoRobotSeatInfo;
import im.zego.minigameengine.ZegoStartGameConfig;
import im.zego.minigameengine.ZegoUserSeatInfo;
import im.zego.minigameenginedemo.R;
import im.zego.minigameenginedemo.RTCEngineManager;
import im.zego.minigameenginedemo.ZegoConstants;
import im.zego.minigameenginedemo.audio.AudioPlayer;
import im.zego.minigameenginedemo.model.RoomGameInfo;
import im.zego.zegoexpress.entity.ZegoRoomExtraInfo;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link ChatFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class ChatFragment extends LiveFragmentBase {
    private static final String TAG = "LiveFragment";
    public static final String ROOM_GAME_INFO = "game_info"; // 房间附加消息的key
    private TextView startHostGameTv;
    private MemberListAdapter memberListAdapter = new MemberListAdapter();
    private RecyclerView memberRecyclerView;
    private View memberListLayout;
    private ZegoGameState gameEvent = ZegoGameState.ZegoGameIdle;

    public ChatFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @return A new instance of fragment LiveFragment.
     */
    public static ChatFragment newInstance() {
        ChatFragment fragment = new ChatFragment();
        return fragment;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // 创建视图
        View view = super.onCreateView(inflater, container, savedInstanceState);
        // 获取流信息列表
        ArrayList<RTCEngineManager.StreamInfo> streamList = rtcManager.getStreamInfoList();
        // 获取成员列表布局
        memberListLayout = view.findViewById(R.id.member_list_layout);
        // 获取成员列表的RecyclerView
        memberRecyclerView = view.findViewById(R.id.member_list_recycler_view);
        // 获取开始主持游戏的按钮
        startHostGameTv = view.findViewById(R.id.start_host_game);
        // 创建线性布局管理器，并设置为水平方向
        LinearLayoutManager layoutManager = new LinearLayoutManager(requireContext());
        layoutManager.setOrientation(LinearLayoutManager.HORIZONTAL);
        // 设置RecyclerView的布局管理器
        memberRecyclerView.setLayoutManager(layoutManager);
        // 设置RecyclerView的适配器
        memberRecyclerView.setAdapter(memberListAdapter);

        // 设置开始主持游戏按钮的点击事件
        startHostGameTv.setOnClickListener(v -> {
            // 创建输入布局
            ViewGroup inputLayout = (ViewGroup) LayoutInflater.from(requireContext()).inflate(R.layout.select_member_dialog, null);
            // 获取成员列表的RecyclerView
            RecyclerView recyclerView = inputLayout.findViewById(R.id.member_list_recyclerview);
            // 获取最小积分的单选组
            RadioGroup minGamePointRadioGroup = inputLayout.findViewById(R.id.player_spin_group);
            // 获取玩家数量的单选组
            RadioGroup playerNumRadioGroup = inputLayout.findViewById(R.id.player_number_group);

            RadioButton taxTypeForAll = inputLayout.findViewById(R.id.tax_type_1);
            RadioButton taxRate0 = inputLayout.findViewById(R.id.tax_rate_0);
            RadioButton taxRate1 = inputLayout.findViewById(R.id.tax_rate_1);

            // 获取是否使用机器人的单选按钮
            RadioButton useRobot = inputLayout.findViewById(R.id.yes);
            // 创建垂直布局管理器，并设置为垂直方向
            LinearLayoutManager verticalLayoutManager = new LinearLayoutManager(requireContext());
            verticalLayoutManager.setOrientation(LinearLayoutManager.VERTICAL);
            // 设置RecyclerView的布局管理器
            recyclerView.setLayoutManager(verticalLayoutManager);

            // 创建选择成员列表的适配器
            SelectMemberListAdapter selectMemberListAdapter = new SelectMemberListAdapter();
            // 获取RTC用户列表
            List<RTCEngineManager.RTCUser> newList = new ArrayList<>(rtcManager.getRtcUserList());
            // 创建当前用户
            RTCEngineManager.RTCUser me = new RTCEngineManager.RTCUser(viewModel.userID, viewModel.userName);
            // 根据是否为主持人设置用户类型
            if (viewModel.isHost) {
                me.type = 0;
            } else {
                me.type = 1;
            }
            // 将当前用户添加到列表中
            newList.add(me);
            // 设置选择成员列表的最大选择数量
            selectMemberListAdapter.setMaxSelectCount(maxPlayerNum);
            // 设置选择成员列表的数据列表
            selectMemberListAdapter.setDataList(newList);
            // 设置RecyclerView的适配器
            recyclerView.setAdapter(selectMemberListAdapter);

            // 设置游戏人数单选组的选中改变监听器
            playerNumRadioGroup.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
                @Override
                public void onCheckedChanged(RadioGroup group, int checkedId) {
                    // 设置选择成员列表的最大选择数量
                    selectMemberListAdapter.setMaxSelectCount(maxPlayerNum);
                    // 重置选择成员列表
                    selectMemberListAdapter.reset();
                }
            });

            // 获取当前游戏的信息
            ZegoGameInfoDetail gameInfoDetail = gameMap.get(currentGameID);
            if (gameInfoDetail != null) {
                // 更新游戏参数
                updateGameParams(gameInfoDetail, minGamePointRadioGroup, playerNumRadioGroup);
            } else {
                // 获取游戏信息
                miniGameEngine.getGameInfo(currentGameID, (errorCode, gameInfoDetail1) -> {
                    if (errorCode != 0) {
                        showToast(String.format(getString(R.string.get_game_info_failed), ZegoConstants.getErrorMsg(errorCode)));
                    }
                    if (gameInfoDetail1 != null) {
                        gameMap.put(currentGameID, gameInfoDetail1);
                    }
                    updateGameParams(gameInfoDetail1, minGamePointRadioGroup, playerNumRadioGroup);
                });
            }

            // 创建对话框构建器
            AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext(), android.R.style.Theme_Translucent_NoTitleBar);
            // 设置对话框的视图
            dialogBuilder.setView(inputLayout);
            // 创建对话框
            AlertDialog dialog = dialogBuilder.create();

            // 获取开始游戏按钮
            TextView startGameBtn = inputLayout.findViewById(R.id.start_game_btn);
            // 设置开始游戏按钮的点击事件
            startGameBtn.setOnClickListener(v1 -> {
                int playerNum = -1;

                if (playerNumRadioGroup.getChildCount() > 0) {
                    for (int i = 0; i < playerNumRadioGroup.getChildCount(); i++) {
                        RadioButton child = (RadioButton) playerNumRadioGroup.getChildAt(i);
                        if (child.isChecked()) {
                            playerNum = Integer.valueOf(child.getText().toString());
                            break;
                        }
                    }
                }

                long minGamePoint = -1;
                if (minGamePointRadioGroup.getChildCount() > 0) {
                    for (int i = 0; i < minGamePointRadioGroup.getChildCount(); i++) {
                        RadioButton child = (RadioButton) minGamePointRadioGroup.getChildAt(i);
                        if (child.isChecked()) {
                            minGamePoint = Long.valueOf(child.getText().toString());
                            break;
                        }
                    }
                }

                // 获取税收类型
                ZegoGameTaxType taxType = taxTypeForAll.isChecked() ? ZegoGameTaxType.ZegoGameTaxTypeForAll : ZegoGameTaxType.ZegoGameTaxTypeForWinner;
                // 获取税率
                int taxRate = taxRate0.isChecked() ? 0 : (taxRate1.isChecked() ? 1 : 2);
                // 更新最大玩家数量
                maxPlayerNum = playerNum;

                // 获取是否使用机器人
                boolean isUseRobot = useRobot.isChecked();
                // 获取选择的用户列表
                List<RTCEngineManager.RTCUser> selectUserList = selectMemberListAdapter.getSelectUserList();
                if (!isUseRobot && selectUserList.size() < maxPlayerNum) {
                    showToast(getString(R.string.select_player_not_enough));
                    return;
                } else if (selectUserList.size() > maxPlayerNum) {
                    showToast(getString(R.string.select_player_exceed_limit));
                    return;
                }

                // 构造用户列表
                List<ZegoUserSeatInfo> userList = new ArrayList<>();
                int seatIndex = 0;
                for (RTCEngineManager.RTCUser user : selectUserList) {
                    userList.add(new ZegoUserSeatInfo(user.userID, seatIndex++));
                }

                List<ZegoRobotSeatInfo> robotList = null;
                if (selectUserList.size() < maxPlayerNum) {
                    robotList = new ArrayList<>();
                    for (int i = seatIndex; i < maxPlayerNum; i++) {
                        String userName = ZegoConstants.getRandomName();
                        String userAvatar = ZegoConstants.getRandomAvatar();
                        ZegoRobotSeatInfo robotSeatInfo = new ZegoRobotSeatInfo(userName, i, userAvatar, ZegoConstants.rand.nextInt(5), 12000);
                        robotList.add(robotSeatInfo);
                    }
                }

                // 创建游戏配置
                ZegoStartGameConfig gameConfig = new ZegoStartGameConfig();
                gameConfig.taxType = taxType;
                gameConfig.taxRate = taxRate;
                gameConfig.minGamePoint = 0;
//                gameConfig.customConfig.put("isRobotForcedStart", 1);
                // 开始游戏
                miniGameEngine.startGame(gameConfig, userList, robotList, ((errorCode, object) -> {

                }));
                dialog.dismiss();
            });

            // 获取对话框的窗口
            Window window = dialog.getWindow();
            // 设置窗口动画
            window.setWindowAnimations(R.style.AnimBottom);
            // 设置窗口的宽度和高度
            window.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
            // 设置窗口的位置
            window.setGravity(Gravity.BOTTOM);
            // 显示对话框
            dialog.show();
        });

        // 如果是主持人
        if (viewModel.isHost) {
            // 判断是否已经有房主在房间里了。
            for (RTCEngineManager.StreamInfo streamInfo : streamList) {
                if (streamInfo.streamID.endsWith("_host")) {
                    hostID = streamInfo.userID;
                    rtcManager.stopPublishStream();
                    rtcManager.logoutRoom();
                    // 先判断是否已经有达到上限的主播在房间里了
                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext());
                    dialogBuilder.setTitle(R.string.notify).setMessage(R.string.live_room_is_occupied).setPositiveButton(R.string.confirm, (dialog, which) -> {
                        requireActivity().onBackPressed();
                    });
                    dialogBuilder.setOnCancelListener(dialog -> requireActivity().onBackPressed());
                    dialogBuilder.show();
                    return view;
                }
            }
        }

        handlePublishStream();
        if (viewModel.isHost) {
            // 设置房间附加消息为空，以确保其他人进来能拿到消息，判断房主身份
            rtcManager.setRoomExtraInfo(viewModel.roomID, ROOM_GAME_INFO, "");
        } else {
            // 获取房间的附加消息列表
            ArrayList<ZegoRoomExtraInfo> roomExtraInfoList = rtcManager.getRoomExtraInfoList();
            if (roomExtraInfoList != null && roomExtraInfoList.size() > 0) {
                // 从房间附加消息获取房间的游戏信息，如果有则加载游戏
                // 解析游戏的id和房间id
                for (ZegoRoomExtraInfo roomExtraInfo : roomExtraInfoList) {
                    hostID = roomExtraInfo.updateUser.userID;
                    memberListAdapter.setHostID(hostID);
                    if (roomExtraInfo.key.equals(ROOM_GAME_INFO)) {
                        String value = roomExtraInfo.value;
                        if (!TextUtils.isEmpty(value)) {
                            roomGameInfo = gson.fromJson(value, RoomGameInfo.class);
                            miniGameEngine.setGameContainer(gameLayout);
                            currentGameID = roomGameInfo.gameID;
                            audioPlayer = new AudioPlayer();
                            // 加载游戏
                            loadGame(currentGameID, ZegoGameMode.fromValue(roomGameInfo.gameMode), roomGameInfo.roomID, roomGameInfo.isSoundCustomPlay);
                        }
                    }
                }
            }
        }

        // 获取新的RTC用户列表
        List<RTCEngineManager.RTCUser> newList = new ArrayList<>(rtcManager.getRtcUserList());
        for (RTCEngineManager.RTCUser rtcUser : newList) {
            if (rtcUser.userID.equals(hostID)) {
                rtcUser.type = 0;
                break;
            }
        }
        // 创建当前用户
        RTCEngineManager.RTCUser me = new RTCEngineManager.RTCUser(viewModel.userID, viewModel.userName);
        if (viewModel.isHost) {
            memberListAdapter.setHostID(viewModel.userID);
            me.type = 0;
        } else {
            me.type = 1;
        }
        // 将当前用户添加到列表中
        newList.add(me);
        // 设置成员列表的数据列表
        memberListAdapter.setDataList(newList);
        // 设置成员列表布局可见
        memberListLayout.setVisibility(View.VISIBLE);

        // 处理流添加事件
        onStreamAdd(rtcManager.getStreamInfoList());
        // 更新按钮的可见性
        updateButtonVisibility();
        return view;
    }

    protected void handlePublishStream() {
        // 在语聊房中，房主和观众都推流
        rtcManager.startPublishStream(viewModel.isHost);
    }

    @Override
    protected void onHostIDUpdate(String hostID) {
        // 设置主持人ID
        memberListAdapter.setHostID(hostID);
    }

    @Override
    protected int getLayoutResourceID() {
        // 返回布局资源ID
        return R.layout.fragment_chat;
    }

    @Override
    public void onDestroyView() {
        // 停止推流
        rtcManager.stopPublishStream();
        super.onDestroyView();
    }

    @Override
    protected void updateButtonVisibility() {
        if (viewModel.isHost) {
            if (isPlayingGame) {
                // 如果正在游戏中，隐藏开始游戏按钮，显示关闭游戏按钮
                playGameBtn.setVisibility(View.GONE);
                closeGameBtn.setVisibility(View.VISIBLE);
                if (gameEvent == ZegoGameState.ZegoGameIdle) {
                    // 如果游戏状态为闲置，显示开始主持游戏按钮
                    startHostGameTv.setVisibility(View.VISIBLE);
                } else {
                    startHostGameTv.setVisibility(View.GONE);
                }
            } else {
                // 如果不在游戏中，显示开始游戏按钮，隐藏关闭游戏按钮
                playGameBtn.setVisibility(View.VISIBLE);
                closeGameBtn.setVisibility(View.GONE);
                startHostGameTv.setVisibility(View.GONE);
            }
            // 隐藏离开游戏按钮和播放秀场游戏按钮
            leaveGameBtn.setVisibility(View.GONE);
            playShowGameBtn.setVisibility(View.INVISIBLE);
        } else {
            if (isPlayingGame) {
                // 如果正在游戏中，隐藏播放秀场游戏按钮，显示离开游戏按钮
                playShowGameBtn.setVisibility(View.INVISIBLE);
                leaveGameBtn.setVisibility(View.VISIBLE);
            } else {
                // 如果不在游戏中，显示播放秀场游戏按钮，隐藏离开游戏按钮
                playShowGameBtn.setVisibility(View.VISIBLE);
                leaveGameBtn.setVisibility(View.GONE);
            }
            // 隐藏开始主持游戏按钮、关闭游戏按钮和开始游戏按钮
            startHostGameTv.setVisibility(View.GONE);
            closeGameBtn.setVisibility(View.GONE);
            playGameBtn.setVisibility(View.GONE);
        }
    }

    @Override
    public void onStreamAdd(ArrayList<RTCEngineManager.StreamInfo> streamInfoList) {
        // 处理流添加事件
        for (RTCEngineManager.StreamInfo streamInfo : streamInfoList) {
            if (streamInfo.streamID.startsWith(RTCEngineManager.STREAM_PREFIX)) {
                if (streamInfo.streamID.equals(rtcManager.publishStreamID)) {
                    continue;
                }

                rtcManager.startPlayingStream(streamInfo.streamID, null);
            }
        }
    }

    @Override
    public void onRoomUserUpdate(List<RTCEngineManager.RTCUser> newUserList) {
        // 处理房间用户更新事件
        List<RTCEngineManager.RTCUser> newList = new ArrayList<>(newUserList);
        for (RTCEngineManager.RTCUser rtcUser : newList) {
            if (rtcUser.userID.equals(hostID)) {
                rtcUser.type = 0;
                break;
            }
        }
        RTCEngineManager.RTCUser me = new RTCEngineManager.RTCUser(viewModel.userID, viewModel.userName);
        if (viewModel.isHost) {
            me.type = 0;
        } else {
            me.type = 1;
        }
        newList.add(me);
        memberListAdapter.setDataList(newList);
    }

    @Override
    public void onGameStateUpdate(ZegoGameState gameState, int reasonCode) {
        // 处理游戏状态更新事件
        this.gameEvent = gameState;
        if (viewModel.isHost && (gameState == ZegoGameState.ZegoGameIdle || gameState == ZegoGameState.ZegoGameOver)) {
            // 如果是主持人且游戏状态为闲置或游戏结束，显示开始主持游戏按钮
            startHostGameTv.setVisibility(View.VISIBLE);
        } else {
            startHostGameTv.setVisibility(View.GONE);
        }
    }

    @Override
    public void onComponentListUpdate(List<ZegoGameComponent> componentList) {
    }

    @Override
    public void onGameLoadStateUpdate(ZegoGameLoadState gameLoadState) {
        super.onGameLoadStateUpdate(gameLoadState);
        // 处理游戏加载状态更新事件
        if (viewModel.isHost && gameLoadState == ZegoGameLoadState.ZegoGameLoaded) {
            // 如果是主持人且游戏加载状态为已加载，显示开始主持游戏按钮
            startHostGameTv.setVisibility(View.VISIBLE);
        } else {
            startHostGameTv.setVisibility(View.GONE);
        }
    }
}