package im.zego.minigameenginedemo.ui.main;

import static im.zego.minigameenginedemo.ui.main.PlayGameFragment.PARAM_GAME_ID;

import android.graphics.Color;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.ContextThemeWrapper;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;

import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.lifecycle.ViewModelProvider;

import java.util.HashMap;

import im.zego.minigameengine.IZegoCommonCallback;
import im.zego.minigameengine.cloudgame.IZegoCloudGameEngineHandler;
import im.zego.minigameengine.cloudgame.ZegoCloudGameEngine;
import im.zego.minigameenginedemo.KeyCenter;
import im.zego.minigameenginedemo.R;
import im.zego.minigameenginedemo.RTCEngineManager;
import im.zego.minigameenginedemo.ZegoConstants;
import im.zego.minigameenginedemo.backend.BackendService;
import im.zego.minigameenginedemo.utils.CommonUtils;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link CloudGameFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class CloudGameFragment extends Fragment {
    private static final String TAG = "CloudGameFragment";
    private String gameID = "TinyLoveWar";
    private String roomID = ZegoConstants.LIVE_ROOM_PREFIX + "live4";
    private String streamID = RTCEngineManager.STREAM_PREFIX + roomID + "_streamID";
    private LinearLayout menuLayout;
    private TextView closeGameBtn;
    private ViewGroup viewGroup;

    private ZegoCloudGameEngine cloudGameEngine = ZegoCloudGameEngine.getInstance();
    private MiniGameMainViewModel viewModel;
    private int roomSelectIndex = 3;
    private String launchGameCode = null;

    private String roomList[] = new String[]{ZegoConstants.LIVE_ROOM_PREFIX + "live1", ZegoConstants.LIVE_ROOM_PREFIX + "live2", ZegoConstants.LIVE_ROOM_PREFIX + "live3",
            ZegoConstants.LIVE_ROOM_PREFIX + "live4", ZegoConstants.LIVE_ROOM_PREFIX + "live5", ZegoConstants.LIVE_ROOM_PREFIX + "live6"};
    public CloudGameFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @return A new instance of fragment CloudGameFragment.
     */
    public static CloudGameFragment newInstance(String gameID) {
        CloudGameFragment fragment = new CloudGameFragment();
        Bundle args = new Bundle();
        args.putString(PARAM_GAME_ID, gameID);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // 获取传递的参数
        if (getArguments() != null) {
            gameID = getArguments().getString(PARAM_GAME_ID);
        }
        // 创建ViewModel实例
        viewModel = new ViewModelProvider(requireActivity()).get(MiniGameMainViewModel.class);
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        ZegoCloudGameEngine.destroyInstance();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        RTCEngineManager.getInstance().logoutRoom();
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_cloud_game, container, false);
        View menuZone = view.findViewById(R.id.menu_zone);
        menuLayout = view.findViewById(R.id.command_list_layout);
        view.findViewById(R.id.show_menu_btn).setOnClickListener(v -> {
            if (menuZone.getVisibility() == View.VISIBLE) {
                menuZone.setVisibility(View.GONE);
            } else {
                menuZone.setVisibility(View.VISIBLE);
            }
        });

        closeGameBtn = view.findViewById(R.id.close_game_btn);
        closeGameBtn.setOnClickListener(v -> {
            cloudGameEngine.stopPublishCloudGameStream(gameID, roomID, streamID, (errorCode, object) -> {
                cloudGameEngine.stopCloudGame(gameID, roomID, (errorCode2, object2) -> {
                    FragmentActivity fragmentActivity = requireActivity();
                    if (fragmentActivity != null) {
                        fragmentActivity.onBackPressed();
                    }
                });
            });
        });

        initMenu();
        cloudGameEngine.setCloudGameEngineHandler(new IZegoCloudGameEngineHandler() {
            @Override
            public void onCloudGameError(int errorCode, String message) {
                showToast("onCloudGameError:" + errorCode + ", " + message);
            }

            @Override
            public void onCloudGameFirstFrame() {
                showToast("onCloudGameFirstFrame");
            }
        });
        viewGroup = view.findViewById(R.id.game_container);
        ZegoCloudGameEngine.getInstance().setCloudGameContainer(viewGroup);
        return view;
    }

    private void initMenu() {
//        addCommand("getCloudGameList", value -> {
//            ZegoMiniGameEngine.getInstance().getGameList(ZegoGameMode.ZegoGameModeCloudGame, (errorCode, gameList) -> {
//                Log.i(TAG, "getGameList errorCode:" + errorCode + ", gameList:" + gameList);
//                cloudGameList = gameList;
//                if (cloudGameList != null && cloudGameList.size() > 0) {
//                    gameID = cloudGameList.get(0).gameID;
//                    showToast("gameID:" + gameID);
//                } else {
//                    showToast("no game");
//                }
//            });
//        });

        ArrayAdapter<Object> roomAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, roomList);
        roomAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        addCommandWithSelector("selectRoom", roomAdapter, roomSelectIndex, value -> {
            roomID = (String)value;
        });

        addCommand("getLaunchGameCode", value -> {
            BackendService.getLaunchCode(viewModel.userID, KeyCenter.appID, gameID, roomID, viewModel.userAvatar, 1, (errorCode, launchCode) -> {
                if (errorCode == 0) {
                    launchGameCode = launchCode;
                }

                showToast("getLaunchGameCode errorCode:" + errorCode + ", launchGameCode:" + launchCode);
            });
        });

        addCommand("clearLaunchGameCode", value -> {
            launchGameCode = null;
        });

        addCommandWithInput("startCloudGame", "", value -> {
            // 要有人登录进房间，否则转推云渲染流会失败
            if (!RTCEngineManager.getInstance().isInRoom()) {
                RTCEngineManager.getInstance().loginRoom(roomID, "", viewModel.userID, viewModel.userName, ((errorCode, object) -> {
                }));
            }

            HashMap<String, Object> gameConfig = new HashMap<>();
            /**
             * 命令行启动，增加三个参数：width height auto 分别表示宽度，高度和自动登录。
             * 注：如果有width和height的话，就会禁用分辨率自定义，且width必须和height成对出现，auto只有token存在才可以有效
             * TinyLoveWar.exe token=101312 width=607 height=1000 auto=1
             * 其中"token=101312 width=607 height=1000 auto=1"为运行参数
             */
            gameConfig.put(ZegoCloudGameEngine.KEY_WL_RESOLUTION, "1080x1920");
            gameConfig.put(ZegoCloudGameEngine.KEY_WL_BIT_RATE, 12000);
            gameConfig.put(ZegoCloudGameEngine.KEY_WL_CODEC_TYPE, 18);
            gameConfig.put(ZegoCloudGameEngine.KEY_WL_FPS, 60);
            gameConfig.put("concurrencyType", value);
            if (!TextUtils.isEmpty(launchGameCode)) {
                // runningParameters作为云游戏运行时附带的参数的key。
                gameConfig.put("runningParameters", "token=" + launchGameCode + " auto=1");
            }

            cloudGameEngine.startCloudGame(gameID, roomID, gameConfig, (errorCode, object) -> {
                showToast("startCloudGameResult:" + ZegoConstants.getErrorMsg(errorCode));
            });
        });

        addCommandWithInput("startCloudGame(WL)", "10000", value -> {
            // 要有人登录进房间，否则转推云渲染流会失败
            if (!RTCEngineManager.getInstance().isInRoom()) {
                RTCEngineManager.getInstance().loginRoom(roomID, "", viewModel.userID, viewModel.userName, ((errorCode, object) -> {
                }));
            }

            Integer bitrate = Integer.parseInt(value);
            HashMap<String, Object> gameConfig = new HashMap<>();
            /**
             * 命令行启动，增加三个参数：width height auto 分别表示宽度，高度和自动登录。
             * 注：如果有width和height的话，就会禁用分辨率自定义，且width必须和height成对出现，auto只有token存在才可以有效
             * TinyLoveWar.exe token=101312 width=607 height=1000 auto=1
             * 其中"token=101312 width=607 height=1000 auto=1"为运行参数
             */
            gameConfig.put(ZegoCloudGameEngine.KEY_WL_RESOLUTION, "1080x1920");
            gameConfig.put(ZegoCloudGameEngine.KEY_WL_BIT_RATE, bitrate);
            gameConfig.put(ZegoCloudGameEngine.KEY_WL_CODEC_TYPE, 18);
            gameConfig.put(ZegoCloudGameEngine.KEY_WL_FPS, 60);
            if (!TextUtils.isEmpty(launchGameCode)) {
                // runningParameters作为云游戏运行时附带的参数的key。
                gameConfig.put("runningParameters", "token=" + launchGameCode + " auto=1");
            }

            cloudGameEngine.startCloudGame(gameID, roomID, gameConfig, (errorCode, object) -> {
                showToast("startCloudGameResult:" + ZegoConstants.getErrorMsg(errorCode));
            });
        });
        addCommand("stopCloudGame", value -> {
            cloudGameEngine.stopCloudGame(gameID, roomID, (errorCode, object) -> {
                showToast("stopCloudGameResult:" + ZegoConstants.getErrorMsg(errorCode));
            });
        });

        addCommand("startPublishGameStream", value -> {
            streamID = RTCEngineManager.STREAM_PREFIX + roomID + "_streamID";
            cloudGameEngine.startPublishCloudGameStream(gameID, roomID, streamID, (errorCode, object) -> {
                showToast("startPublishGameStreamResult:" + ZegoConstants.getErrorMsg(errorCode));
            });
        });

        addCommand("stopPublishGameStream", value -> {
            cloudGameEngine.stopPublishCloudGameStream(gameID, roomID, streamID, (errorCode, object) -> {
                showToast("stopPublishGameStreamResult:" + ZegoConstants.getErrorMsg(errorCode));
            });
        });

        addCommand("getGameVolume", value -> {
            cloudGameEngine.getPlayVolume(new IZegoCommonCallback<Integer>() {
                @Override
                public void onResult(int errorCode, Integer volume) {
                    showToast("getGameVolume:" + volume + ", errorCode:" + ZegoConstants.getErrorMsg(errorCode));
                }
            });
        });

        addCommandWithInput("cloudGameControl", "type:value", value -> {
            int index = value.indexOf(":");
            if (index > 0) {
                cloudGameEngine.controlCloudGame(gameID, roomID, value.substring(0, index), value.substring(index + 1));
                showToast("cloudGameControl type:" + value.substring(0, index) + ", value:" + value.substring(index + 1));
            }
        });

        addCommandWithInput("setGameVolume", "50", value -> {
            int ret = cloudGameEngine.setPlayVolume(Integer.valueOf(value));
            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });

        /**
         * key  keycode
         * w      87
         * a      65
         * s      83
         * d      68
         * 1      49
         * 2      50
         * 3      51
         * 4      52
         * 5      53
         */
        addCommand("send w key", value -> {
            cloudGameEngine.sendKeyboardEvent(87);
        });

        addCommand("send a key", value -> {
            cloudGameEngine.sendKeyboardEvent(65);
        });

        addCommand("send s key", value -> {
            cloudGameEngine.sendKeyboardEvent(83);
        });

        addCommand("send d key", value -> {
            cloudGameEngine.sendKeyboardEvent(68);
        });

        addCommandWithInput("SendKeyEvent", "", value -> {
            try {
                int key = Integer.parseInt(value);
                cloudGameEngine.sendKeyboardEvent(key);
                showToast("send keyboard event:" + key);
            } catch (Exception e) {
                showToast("invalid key code");
            }
        });
    }

    protected void showToast(String msg) {
        CommonUtils.showToast(msg);
    }

    // 处理添加功能菜单 BEGIN
    interface IRunFunc {
        void onRunFunc(String value);
    }

    interface IRunFunc2 {
        void onRunFunc(Object value);
    }

    private void addCommand(String content, LiveFragmentBase.IRunFunc runFunc) {
        addCommand(menuLayout, content, runFunc);
    }

    private void addCommandWithInput(String content, LiveFragmentBase.IRunFunc runFunc) {
        addCommandWithInput(menuLayout, content, runFunc);
    }

    private void addCommandWithInput(String content, String defaultText, LiveFragmentBase.IRunFunc runFunc) {
        addCommandWithInput(menuLayout, content, defaultText, runFunc);
    }

    private void addCommand(LinearLayout layout, String content, LiveFragmentBase.IRunFunc runFunc) {
        Button button = new Button(new ContextThemeWrapper(requireContext(), R.style.testButtonStyle));
        button.setText(content);
        button.setOnClickListener(v -> {
            runFunc.onRunFunc("");
        });
        layout.addView(button);
    }

    private void addCommandWithInput(LinearLayout layout, String content, LiveFragmentBase.IRunFunc runFunc) {
        Button button = new Button(new ContextThemeWrapper(requireContext(), R.style.testButtonStyle));
        button.setText(content);
        EditText editText = new EditText(new ContextThemeWrapper(requireContext(), R.style.testButtonStyle));
        editText.setBackground(getResources().getDrawable(R.drawable.edittext_bg));
        LinearLayout commandLayout = new LinearLayout(requireContext());
        commandLayout.setOrientation(LinearLayout.HORIZONTAL);
        commandLayout.addView(editText, new ViewGroup.LayoutParams(CommonUtils.dp2px(requireContext(), 100f), CommonUtils.dp2px(requireContext(), 40f)));
        commandLayout.addView(button, new ViewGroup.LayoutParams(CommonUtils.dp2px(requireContext(), 190f), CommonUtils.dp2px(requireContext(), 40f)));
        button.setOnClickListener(v -> {
            runFunc.onRunFunc(editText.getText().toString());
        });
        commandLayout.setPadding(10, 10, 10, 10);
        layout.addView(commandLayout);
    }

    private void addCommandWithInput(LinearLayout layout, String content, String defaultText, LiveFragmentBase.IRunFunc runFunc) {
        Button button = new Button(new ContextThemeWrapper(requireContext(), R.style.testButtonStyle));
        button.setText(content);
        EditText editText = new EditText(new ContextThemeWrapper(requireContext(), R.style.testButtonStyle));
        editText.setSingleLine(true);
        editText.setText(defaultText);
        editText.setBackground(getResources().getDrawable(R.drawable.edittext_bg));
        LinearLayout commandLayout = new LinearLayout(requireContext());
        commandLayout.setOrientation(LinearLayout.HORIZONTAL);
        commandLayout.addView(editText, new ViewGroup.LayoutParams(CommonUtils.dp2px(requireContext(), 100f), CommonUtils.dp2px(requireContext(), 40f)));
        commandLayout.addView(button, new ViewGroup.LayoutParams(CommonUtils.dp2px(requireContext(), 190f), CommonUtils.dp2px(requireContext(), 40f)));
        button.setOnClickListener(v -> {
            runFunc.onRunFunc(editText.getText().toString());
        });
        commandLayout.setPadding(10, 10, 10, 10);
        layout.addView(commandLayout);
    }

    private void addCommandWithSelector(String content, ArrayAdapter<Object> adapter, int selectIndex, LiveFragmentBase.IRunFunc2 runFunc) {
        addCommandWithSelector(menuLayout, content, adapter, selectIndex, runFunc);
    }

    private void addCommandWithSelector(LinearLayout layout, String content, ArrayAdapter<Object> adapter, int selectIndex, LiveFragmentBase.IRunFunc2 runFunc) {
        TextView text = new TextView(layout.getContext());
        text.setText(content);
        Spinner spinner = new Spinner(layout.getContext());
        spinner.setAdapter(adapter);
        spinner.setSelection(selectIndex);
        spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                Object selectItem = adapter.getItem(position);
                runFunc.onRunFunc(selectItem);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
        LinearLayout commandLayout = new LinearLayout(requireContext());
        commandLayout.setBackgroundColor(Color.parseColor("#ffffff"));
        commandLayout.setOrientation(LinearLayout.HORIZONTAL);
        commandLayout.addView(text, new ViewGroup.LayoutParams(CommonUtils.dp2px(requireContext(), 100f), CommonUtils.dp2px(requireContext(), 40f)));
        commandLayout.addView(spinner, new ViewGroup.LayoutParams(CommonUtils.dp2px(requireContext(), 190f), CommonUtils.dp2px(requireContext(), 40f)));
        commandLayout.setPadding(10, 10, 10, 10);
        layout.addView(commandLayout);
    }
    // 处理添加功能菜单 END
}