package im.zego.minigameenginedemo.ui.main;

import android.annotation.SuppressLint;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;

import java.util.List;

import im.zego.minigameenginedemo.R;
import im.zego.minigameenginedemo.ZegoConstants;
import im.zego.minigameenginedemo.model.DemoGameInfo;

public class DemoGameListAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    protected List<DemoGameInfo> dataList;
    private IButtonClickListener buttonClickListener;
    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.demo_game_item_layout, parent, false);
        GameViewHolder gameViewHolder = new GameViewHolder(view);
        return gameViewHolder;
    }

    @SuppressLint("StringFormatInvalid")
    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        GameViewHolder gameViewHolder = (GameViewHolder)holder;
        DemoGameInfo gameInfo = dataList.get(position);
        gameViewHolder.itemTv.setText(gameInfo.gameInfo.gameName);
        gameViewHolder.gamePointTv.setText(gameInfo.gamePoint + "");
        gameViewHolder.exchangeBtn.setText(String.format(holder.itemView.getContext().getString(R.string.exchange_x_game_point),  ZegoConstants.EXCHANGE_POINT_NUMBER));
        gameViewHolder.preloadBtn.setText(String.format(holder.itemView.getContext().getString(R.string.preload_game), gameInfo.downloadProgress));
        Glide.with(gameViewHolder.itemImage).load(gameInfo.gameInfo.thumbnail).into(gameViewHolder.itemImage);
        gameViewHolder.itemImage.setOnLongClickListener(v -> {
            if (buttonClickListener != null) {
                buttonClickListener.onLongClickIcon(position, gameViewHolder, gameInfo);
            }
            return false;
        });
        gameViewHolder.exchangeBtn.setOnClickListener(v -> {
            if (buttonClickListener != null) {
                buttonClickListener.onClickExchangeButton(position, gameViewHolder, gameInfo);
            }
        });

        gameViewHolder.preloadBtn.setOnClickListener(v -> {
            if (buttonClickListener != null) {
                buttonClickListener.onClickPreloadButton(position, gameViewHolder, gameInfo);
            }
        });

    }

    @Override
    public int getItemCount() {
        return dataList != null ? dataList.size() : 0;
    }

    public static class GameViewHolder extends RecyclerView.ViewHolder {
        public TextView itemTv;
        public ImageView itemImage;
        public TextView gamePointTv;
        public TextView exchangeBtn;
        public TextView preloadBtn;
        public GameViewHolder(@NonNull View itemView) {
            super(itemView);
            itemTv = itemView.findViewById(R.id.game_name_tv);
            itemImage = itemView.findViewById(R.id.game_image);
            gamePointTv = itemView.findViewById(R.id.game_point_tv);
            exchangeBtn = itemView.findViewById(R.id.exchange_point_count_btn);
            preloadBtn = itemView.findViewById(R.id.preload_btn);
        }
    }

    interface IButtonClickListener {
        void onLongClickIcon(int position, GameViewHolder viewHolder, DemoGameInfo info);
        void onClickExchangeButton(int position, GameViewHolder viewHolder, DemoGameInfo info);
        void onClickPreloadButton(int position, GameViewHolder viewHolder, DemoGameInfo info);
    }

    public void setDataList(List<DemoGameInfo> dataList) {
        this.dataList = dataList;
        notifyDataSetChanged();
    }

    public void setButtonClickListener(IButtonClickListener listener) {
        buttonClickListener = listener;
    }
}
