package im.zego.minigameenginedemo.ui.main;


import android.util.Log;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import im.zego.minigameengine.ZegoGameState;
import im.zego.minigameenginedemo.RTCEngineManager;
import im.zego.zegoexpress.constants.ZegoStreamEvent;

/**
 * Use the {@link GameRoomFragment#newInstance} factory method to
 * create an instance of this fragment.
 *
 */
public class GameRoomFragment extends ChatFragment {
    private static final String TAG = "GameRoomFragment";
    private boolean isPublishedStream = false;
    private HashMap<String, Boolean> speakerStateMap = new HashMap<>();
    public static GameRoomFragment newInstance() {
        GameRoomFragment fragment = new GameRoomFragment();
        return fragment;
    }

    public GameRoomFragment() {
        // Required empty public constructor
        super();
        rtcManager.initAudioGameConfig();
    }

    @Override
    protected void handlePublishStream() {
        // 在游戏房中，房主一开始就推流
        if (viewModel.isHost) {
            rtcManager.startPublishStream(true);
            isPublishedStream = true;
        }
    }

    @Override
    public void onGameStateUpdate(ZegoGameState gameState) {
        if (gameState == ZegoGameState.ZegoGameOver) {
            if (isPublishedStream && !viewModel.isHost) {
                rtcManager.stopPublishStream();
            }
            speakerStateMap.clear();
        }
    }

    @Override
    public void onMicStateChange(boolean isMute) {
        rtcManager.muteMicrophone(isMute);
        if (!isPublishedStream) {
            rtcManager.startPublishStream(viewModel.isHost);
            isPublishedStream = true;
        }
    }

    @Override
    public void onSpeakerStateChange(List<String> userIDList, boolean isMute) {
        Log.i(TAG, "[onSpeakerStateChange] userIDList: " + userIDList.toArray() + ", isMute:" + isMute);
        for (String userID: userIDList) {
            speakerStateMap.put(userID, isMute);
            if (!userID.equals(viewModel.userID)) {
                rtcManager.muteSpeaker(userID, isMute);
            }
        }

    }

    @Override
    public void onStreamAdd(ArrayList<RTCEngineManager.StreamInfo> streamInfoList) {
        // 处理流添加事件
        for (RTCEngineManager.StreamInfo streamInfo : streamInfoList) {
            if (streamInfo.streamID.startsWith(RTCEngineManager.STREAM_PREFIX)) {
                if (streamInfo.streamID.equals(rtcManager.publishStreamID)) {
                    continue;
                }

                // 默认拉流就把这条流静音，是否打开依赖游戏的通知
                if (speakerStateMap.containsKey(streamInfo.userID)) {
                    rtcManager.muteSpeaker(streamInfo.userID, speakerStateMap.get(streamInfo.userID));
                } else {
                    rtcManager.muteSpeaker(streamInfo.userID, true);
                }

                rtcManager.startPlayingStream(streamInfo.streamID, null);
            }
        }
    }

    @Override
    public void onPublisherStreamEvent(ZegoStreamEvent eventID, String streamID, String extraInfo) {
        if (eventID == ZegoStreamEvent.RETRY_PUBLISH_SUCCESS) {
            miniGameEngine.refreshMicAndSpeakerState();
        }
    }
}