package im.zego.minigameenginedemo.ui.main;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.TextureView;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.widget.GridLayout;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;

import java.util.ArrayList;
import java.util.List;

import im.zego.minigameengine.ZegoGameComponent;
import im.zego.minigameengine.ZegoGameMode;
import im.zego.minigameengine.ZegoMiniGameEngine;
import im.zego.minigameenginedemo.R;
import im.zego.minigameenginedemo.RTCEngineManager;
import im.zego.minigameenginedemo.audio.AudioPlayer;
import im.zego.minigameenginedemo.model.LiveModel;
import im.zego.minigameenginedemo.model.RoomGameInfo;
import im.zego.zegoexpress.entity.ZegoRoomExtraInfo;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link LiveFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class LiveFragment extends LiveFragmentBase {
    private static final String TAG = "LiveFragment";
    private TextureView hostTextureView;
    private TextView joinLiveBtn;
    private TextView leaveLiveBtn;
    private ViewGroup streamLayout;
    private ArrayList<ViewGroup> streamViewList = new ArrayList<>();
    private ZegoMiniGameEngine miniGameEngine = ZegoMiniGameEngine.getInstance();
    private boolean isJoinLive = false;
    private ViewGroup myStreamView;

    public LiveFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @return A new instance of fragment LiveFragment.
     */
    public static LiveFragment newInstance() {
        LiveFragment fragment = new LiveFragment();
        return fragment;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // 创建视图
        View view = super.onCreateView(inflater, container, savedInstanceState);
        // 获取流信息列表
        ArrayList streamList = rtcManager.getStreamInfoList();
        // 获取流布局
        streamLayout = view.findViewById(R.id.stream_layout);

        // 加入直播按钮
        joinLiveBtn = view.findViewById(R.id.join_live_btn);
        joinLiveBtn.setOnClickListener(v ->  {
            // 判断是否有主播开始直播
            if (rtcManager.getStreamInfoList().size() == 0) {
                showToast(getString(R.string.host_not_start_live));
                return;
            } else if (!isJoinLive && rtcManager.getStreamInfoList().size() >= 2) {
                showToast(getString(R.string.reach_limit_of_open_mic));
                return;
            }

            // 如果未加入直播，则创建自己的流视图并加入布局中
            if (!isJoinLive) {
                myStreamView = (ViewGroup) LayoutInflater.from(requireContext()).inflate(R.layout.pk_stream_view, null);
                hostTextureView = myStreamView.findViewById(R.id.stream_texture_view);
                TextView textView = myStreamView.findViewById(R.id.room_info);
                textView.setText(viewModel.userName);
                myStreamView.findViewById(R.id.close_pk_view_btn).setVisibility(View.GONE);
                GridLayout.LayoutParams params = new GridLayout.LayoutParams();
                params.width = 0;
                params.height = 0;
                params.rowSpec = GridLayout.spec(0, 1, 1);
                params.columnSpec = GridLayout.spec(0, 1, 1);
                streamLayout.addView(myStreamView, params);
                streamViewList.add(myStreamView);

                // 开始发布流
                rtcManager.startPublishStream(viewModel.isHost);
                myStreamView.setTag(rtcManager.publishStreamID);
                rtcManager.startPreview(hostTextureView);
            }

            // 切换加入直播状态，刷新主播视图，更新按钮可见性
            isJoinLive = !isJoinLive;
            refreshHostView();
            updateButtonVisibility();
        });

        // 离开直播按钮
        leaveLiveBtn = view.findViewById(R.id.leave_live_btn);
        leaveLiveBtn.setOnClickListener(v -> {
            // 如果已加入直播，则移除自己的流视图并停止发布流
            if (isJoinLive) {
                streamLayout.removeView(myStreamView);
                streamViewList.remove(myStreamView);
                rtcManager.stopPublishStream();
            }
            // 切换加入直播状态，刷新主播视图，更新按钮可见性
            isJoinLive = !isJoinLive;
            refreshHostView();
            updateButtonVisibility();
        });

        // 如果是主播身份
        if (viewModel.isHost) {
            if (streamList.size() > 0) {
                // 先判断是否已经有达到上限的主播在房间里了
                AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext());
                dialogBuilder.setTitle(R.string.notify).setMessage(R.string.live_room_is_occupied).setPositiveButton(R.string.confirm, (dialog, which) -> {
                    requireActivity().onBackPressed();
                });
                dialogBuilder.setOnCancelListener(dialog -> requireActivity().onBackPressed());
                dialogBuilder.show();
                return view;
            }
        }

        // 如果是主播身份
        if (viewModel.isHost) {
            // 开始发布流并设置房间附加消息为空
            rtcManager.startPublishStream(true);
            rtcManager.setRoomExtraInfo(viewModel.roomID, ROOM_GAME_INFO, "");
            // 创建主流视图并加入布局中
            ViewGroup mainStreamView = (ViewGroup) LayoutInflater.from(requireContext()).inflate(R.layout.pk_stream_view, null);
            hostTextureView = mainStreamView.findViewById(R.id.stream_texture_view);
            TextView textView = mainStreamView.findViewById(R.id.room_info);
            textView.setText(viewModel.userName);
            mainStreamView.findViewById(R.id.close_pk_view_btn).setVisibility(View.GONE);
            GridLayout.LayoutParams params = new GridLayout.LayoutParams();
            params.width = 0;
            params.height = 0;
            params.rowSpec = GridLayout.spec(0, 1, 1);
            params.columnSpec = GridLayout.spec(0, 1, 1);
            streamLayout.addView(mainStreamView, params);
            streamViewList.add(mainStreamView);

            mainStreamView.setTag(rtcManager.publishStreamID);
            rtcManager.startPreview(hostTextureView);

        } else {
            // 如果是观众身份
            ArrayList<ZegoRoomExtraInfo> roomExtraInfoList = rtcManager.getRoomExtraInfoList();
            if (roomExtraInfoList != null && roomExtraInfoList.size() > 0) {
                // 从房间附加消息获取房间的游戏信息，如果有则加载游戏
                for (ZegoRoomExtraInfo roomExtraInfo : roomExtraInfoList) {
                    if (roomExtraInfo.key.equals(ROOM_GAME_INFO)) {
                        String value = roomExtraInfo.value;
                        if (!TextUtils.isEmpty(value)) {
                            roomGameInfo = gson.fromJson(value, RoomGameInfo.class);
                            miniGameEngine.setGameContainer(gameLayout);
                            currentGameID = roomGameInfo.gameID;
                            audioPlayer = new AudioPlayer();
                            // 加载游戏
                            loadGame(currentGameID, ZegoGameMode.fromValue(roomGameInfo.gameMode), roomGameInfo.roomID, roomGameInfo.isSoundCustomPlay);
                        }
                    }
                }
            }
        }

        // 添加流信息到布局中，更新按钮可见性
        onStreamAdd(rtcManager.getStreamInfoList());
        updateButtonVisibility();
        return view;
    }

    @Override
    protected int getLayoutResourceID() {
        return R.layout.fragment_live;
    }

    @Override
    protected void updateButtonVisibility() {
        // 更新按钮的可见性
        if (viewModel.isHost) {
            // 如果是主播
            if (isPlayingGame) {
                playGameBtn.setVisibility(View.GONE);
                closeGameBtn.setVisibility(View.VISIBLE);
            } else {
                playGameBtn.setVisibility(View.VISIBLE);
                closeGameBtn.setVisibility(View.GONE);
            }
            leaveGameBtn.setVisibility(View.GONE);
            playShowGameBtn.setVisibility(View.INVISIBLE);
            joinLiveBtn.setVisibility(View.INVISIBLE);
            leaveLiveBtn.setVisibility(View.INVISIBLE);
        } else {
            // 如果是观众
            if (isPlayingGame) {
                playShowGameBtn.setVisibility(View.INVISIBLE);
                leaveGameBtn.setVisibility(View.VISIBLE);
            } else {
                playShowGameBtn.setVisibility(View.VISIBLE);
                leaveGameBtn.setVisibility(View.GONE);
            }
            closeGameBtn.setVisibility(View.GONE);
            playGameBtn.setVisibility(View.GONE);
            if (viewModel.roomType == LiveModel.RoomType.LIVE_ROOM) {
                // 如果是直播房间
                if (isJoinLive) {
                    joinLiveBtn.setVisibility(View.INVISIBLE);
                    leaveLiveBtn.setVisibility(View.VISIBLE);
                } else {
                    if (rtcManager.getStreamInfoList().size() == 1) {
                        joinLiveBtn.setVisibility(View.VISIBLE);
                    } else {
                        joinLiveBtn.setVisibility(View.INVISIBLE);
                    }
                    leaveLiveBtn.setVisibility(View.INVISIBLE);
                }
            } else {
                joinLiveBtn.setVisibility(View.INVISIBLE);
                leaveLiveBtn.setVisibility(View.INVISIBLE);
            }
        }
    }

    @Override
    public void onStreamRemove(ArrayList<RTCEngineManager.StreamInfo> streamInfoList) {
        // 当有流被移除时的回调函数
        for (RTCEngineManager.StreamInfo streamInfo : streamInfoList) {
            if (streamInfo.streamID.startsWith(RTCEngineManager.STREAM_PREFIX)) {
                // 如果流的ID以指定前缀开头
                rtcManager.stopPlayingStream(streamInfo.streamID);
                for (ViewGroup viewGroup : streamViewList) {
                    String tag = (String) viewGroup.getTag();
                    if (tag.equals(streamInfo.streamID)) {
                        streamViewList.remove(viewGroup);
                        refreshHostView();
                        updateButtonVisibility();
                        break;
                    }
                }
            }
        }
    }

    @Override
    public void onStreamAdd(ArrayList<RTCEngineManager.StreamInfo> streamInfoList) {
        // 当有流被添加时的回调函数
        for (RTCEngineManager.StreamInfo streamInfo: streamInfoList) {
            if (streamInfo.streamID.startsWith(RTCEngineManager.STREAM_PREFIX)) {
                // 如果流的ID以指定前缀开头
                if (streamInfo.streamID.equals(rtcManager.publishStreamID)) {
                    continue;
                }
                if (viewModel.roomType == LiveModel.RoomType.LIVE_ROOM) {
                    // 如果是直播房间
                    ViewGroup streamView = (ViewGroup) LayoutInflater.from(requireContext()).inflate(R.layout.pk_stream_view, null);
                    TextureView newTextureView = streamView.findViewById(R.id.stream_texture_view);
                    TextView textView = streamView.findViewById(R.id.room_info);
                    textView.setText(streamInfo.userName);
                    streamView.setTag(streamInfo.streamID);
                    streamView.findViewById(R.id.close_pk_view_btn).setVisibility(View.GONE);
                    streamViewList.add(streamView);
                    rtcManager.startPlayingStream(streamInfo.streamID, newTextureView);
                }
            }
        }
        refreshHostView();
        updateButtonVisibility();
    }

    private void refreshHostView() {
        // 刷新主播视图
        if (viewModel.roomType == LiveModel.RoomType.CHAT_ROOM) {
            return;
        }
        int total = streamViewList.size();
        streamLayout.removeAllViews();

        // 根据streamLayout的宽高设定屏幕方向
        int height = streamLayout.getHeight();
        if (height <= 0) {
            ViewTreeObserver viewTreeObserver = streamLayout.getViewTreeObserver();
            viewTreeObserver.addOnGlobalLayoutListener(new ViewTreeObserver.OnGlobalLayoutListener() {
                @Override
                public void onGlobalLayout() {
                    streamLayout.getViewTreeObserver().removeOnGlobalLayoutListener(this);
                    // 如果人数是1个，则streamLayout是全屏，如果是2个以上，则占上边宽：高是16：9 （竖屏）的区域
                    ViewGroup.LayoutParams layoutParams = streamLayout.getLayoutParams();
                    if (total <= 1) {
                        layoutParams.height = ViewGroup.LayoutParams.MATCH_PARENT;
                        layoutParams.width = ViewGroup.LayoutParams.MATCH_PARENT;
                    } else {
                        layoutParams.width = ViewGroup.LayoutParams.MATCH_PARENT;
                        layoutParams.height = streamLayout.getWidth() * 9 / 16;
                    }
                    streamLayout.setLayoutParams(layoutParams);
                    streamLayout.post(() -> {
                        refactorStreamLayout(total);
                    });
                }
            });
        } else {
            // 如果人数是1个，则streamLayout是全屏，如果是2个以上，则占上边宽：高是16：9 （竖屏）的区域
            ViewGroup.LayoutParams layoutParams = streamLayout.getLayoutParams();
            if (total <= 1) {
                layoutParams.height = ViewGroup.LayoutParams.MATCH_PARENT;
                layoutParams.width = ViewGroup.LayoutParams.MATCH_PARENT;
            } else {
                layoutParams.width = ViewGroup.LayoutParams.MATCH_PARENT;
                layoutParams.height = streamLayout.getWidth() * 9 / 16;
            }
            streamLayout.setLayoutParams(layoutParams);
            streamLayout.post(() -> {
                refactorStreamLayout(total);
            });
        }
    }

    private void refactorStreamLayout(int total) {
        // 重新布局流视图
        for (int i = 0; i < total; i++) {
            ViewGroup streamView = streamViewList.get(i);
            String streamID = (String) streamView.getTag();
            TextureView textureView = streamView.findViewById(R.id.stream_texture_view);
            GridLayout.LayoutParams params = new GridLayout.LayoutParams();
            params.width = 0;
            params.height = 0;
            if (total < 5) {
                params.rowSpec = GridLayout.spec(i / 2, 1f);
                params.columnSpec = GridLayout.spec(i % 2, 1f);
            } else {
                params.rowSpec = GridLayout.spec(i / 3, 1f);
                params.columnSpec = GridLayout.spec(i % 3, 1f);
            }

            if (viewModel.isHost && i == 0) {
                // 自己的视频窗口使用预览
                rtcManager.startPreview(textureView);
            } else {
                rtcManager.startPlayingStream(streamID, textureView);
            }

            if (streamLayout.getWidth() < streamLayout.getHeight() && total == 2) {
                // 如果是streamLayout是height > width,则交换row和col
                GridLayout.Spec temp = params.columnSpec;
                params.columnSpec = params.rowSpec;
                params.rowSpec = temp;
            }
            if (streamLayout.indexOfChild(streamView) != -1)  {
                streamLayout.updateViewLayout(streamView, params);
            } else {
                streamLayout.addView(streamView, params);
            }
        }
    }

    @Override
    public void onComponentListUpdate(List<ZegoGameComponent> componentList) {

    }
}