package im.zego.minigameenginedemo.ui.main;

import android.content.res.ColorStateList;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.Log;
import android.view.ContextThemeWrapper;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.RelativeLayout;
import android.widget.Spinner;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.android.material.radiobutton.MaterialRadioButton;
import com.google.gson.Gson;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import im.zego.minigameengine.IZegoGameActionCallback;
import im.zego.minigameengine.IZegoGameEngineHandler;
import im.zego.minigameengine.ZegoGameEngineError;
import im.zego.minigameengine.ZegoGameInfo;
import im.zego.minigameengine.ZegoGameInfoDetail;
import im.zego.minigameengine.ZegoGameLoadState;
import im.zego.minigameengine.ZegoGameMode;
import im.zego.minigameengine.ZegoGamePlayerState;
import im.zego.minigameengine.ZegoGameRobotConfig;
import im.zego.minigameengine.ZegoGameState;
import im.zego.minigameengine.ZegoMiniGameEngine;
import im.zego.minigameenginedemo.KeyCenter;
import im.zego.minigameenginedemo.R;
import im.zego.minigameenginedemo.RTCEngineManager;
import im.zego.minigameenginedemo.ZegoConstants;
import im.zego.minigameenginedemo.audio.AudioPlayer;
import im.zego.minigameenginedemo.model.DemoGameInfo;
import im.zego.minigameenginedemo.model.RoomGameInfo;
import im.zego.minigameenginedemo.utils.CommonUtils;
import im.zego.zegoexpress.constants.ZegoStreamEvent;
import im.zego.zegoexpress.entity.ZegoRoomExtraInfo;

/**
 * A simple {@link Fragment} subclass.
 * create an instance of this fragment.
 */
public abstract class LiveFragmentBase extends Fragment implements RTCEngineManager.IRTCEventListener, IZegoGameEngineHandler {
    private static final String TAG = "LiveFragmentBase";
    public static final String ROOM_GAME_INFO = "game_info"; // 房间附加消息的key
    protected MiniGameMainViewModel viewModel;
    protected LinearLayout menuLayout;
    protected TextView playGameBtn;
    protected TextView closeGameBtn;
    protected TextView leaveGameBtn;
    protected TextView playShowGameBtn;
    protected TextView memberCountTv;
    protected RelativeLayout gameLayout;
    protected TextView loadingTv;
    protected RTCEngineManager rtcManager = RTCEngineManager.getInstance();
    protected ZegoMiniGameEngine miniGameEngine = ZegoMiniGameEngine.getInstance();
    protected Handler handler = new Handler(Looper.getMainLooper());
    protected HashMap<String, ZegoGameInfoDetail> gameMap = new HashMap<>();
    protected String currentGameID = "";
    protected Gson gson = new Gson();
    protected RoomGameInfo roomGameInfo;
    protected AudioPlayer audioPlayer;
    protected boolean isMusicOpen = true;
    protected boolean isSoundEffectOpen = true;
    protected boolean isVibrateOpen = true;
    protected boolean isPlayingGame = false;
    protected int maxPlayerNum = 0;
    protected String hostID = "";
    private boolean isPlayer = false;

    public LiveFragmentBase() {
        // Required empty public constructor
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        viewModel = new ViewModelProvider(requireActivity()).get(MiniGameMainViewModel.class);

        miniGameEngine.setGameEngineHandler(this);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(getLayoutResourceID(), container, false);

        playGameBtn = view.findViewById(R.id.play_game_btn);
        gameLayout = view.findViewById(R.id.game_container);
        leaveGameBtn = view.findViewById(R.id.leave_game_btn);
        playShowGameBtn = view.findViewById(R.id.play_show_game_btn);
        closeGameBtn = view.findViewById(R.id.close_game_btn);
        loadingTv = view.findViewById(R.id.loading_tv);
        memberCountTv = view.findViewById(R.id.member_count_tv);
        memberCountTv.setText(String.format(getString(R.string.online_member), rtcManager.getMemberCount()));

        TextView roomIDTextView = view.findViewById(R.id.room_id_tv);
        roomIDTextView.setText(viewModel.userName + " " + viewModel.roomID);
        view.findViewById(R.id.close_room_btn).setOnClickListener(v -> {
            requireActivity().onBackPressed();
        });

        View menuZone = view.findViewById(R.id.menu_zone);
        menuLayout = view.findViewById(R.id.command_list_layout);
        view.findViewById(R.id.show_menu_btn).setOnClickListener(v -> {
            if (menuZone.getVisibility() == View.VISIBLE) {
                menuZone.setVisibility(View.GONE);
            } else {
                menuZone.setVisibility(View.VISIBLE);
            }
        });

        rtcManager.addRTCEventListener(this);

        playGameBtn.setOnClickListener(v -> {
            ViewGroup dialogLayout = (ViewGroup) LayoutInflater.from(requireContext()).inflate(R.layout.select_mode_dialog, null);
            TextView selectHostGameMode = dialogLayout.findViewById(R.id.host_game_mode);
            TextView selectMatchInGameMode = dialogLayout.findViewById(R.id.match_in_game_mode);
            AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext());
            dialogBuilder.setView(dialogLayout);
            AlertDialog dialog = dialogBuilder.create();

            selectHostGameMode.setOnClickListener(v1 -> {
                onClickPlayGameBtn();
                dialog.dismiss();
            });

            selectMatchInGameMode.setOnClickListener(v1 -> {
                onClickInGameMatchPlayBtn();
                dialog.dismiss();
            });
            dialog.show();
        });

        leaveGameBtn.setOnClickListener(v -> {
            miniGameEngine.unloadGame(true);
        });

        playShowGameBtn.setOnClickListener(v -> {
            // 选择秀场游戏
            onClickPlayShowGameBtn();
        });
        closeGameBtn = view.findViewById(R.id.close_game_btn);
        closeGameBtn.setOnClickListener(v -> {
            miniGameEngine.unloadGame(true);
            rtcManager.setRoomExtraInfo(viewModel.roomID, ROOM_GAME_INFO, "");
            updateButtonVisibility();
        });

        viewModel.refreshGamePoint.observe(requireActivity(), refreshPointObserver);
        initMenu();
        return view;
    }

    protected abstract int getLayoutResourceID();

    private void onClickPlayGameBtn() {
        ViewGroup inputLayout = (ViewGroup) LayoutInflater.from(requireContext()).inflate(R.layout.select_game_dialog, null);
        RecyclerView gameRecyclerView = inputLayout.findViewById(R.id.game_list_recyclerview);
        RadioButton playByNative = inputLayout.findViewById(R.id.play_by_native);
        GridLayoutManager layoutManager = new GridLayoutManager(requireContext(), 4); // 每行4个
        gameRecyclerView.setLayoutManager(layoutManager);
        ChoseGameListAdapter gameListAdapter = new ChoseGameListAdapter();
        gameListAdapter.setSelectable(true);
        gameListAdapter.setDataList(viewModel.hostGameList.getValue());
        gameRecyclerView.setAdapter(gameListAdapter);
        gameListAdapter.setItemClickListener(new OnRecyclerViewItemClickListener() {
            @Override
            public void onItemClick(RecyclerView recyclerView, View view, int position) {
                ZegoGameInfo gameInfo = gameListAdapter.getDataList().get(position);
                ZegoGameInfoDetail gameInfoDetail = gameMap.get(gameInfo.gameID);
                if (gameInfoDetail == null) {
                    miniGameEngine.getGameInfo(gameInfo.gameID, (errorCode, gameInfoDetail1) -> {
                        if (errorCode != 0) {
                            showToast(String.format(getString(R.string.get_game_info_failed), ZegoConstants.getErrorMsg(errorCode)));
                        }
                        if (gameInfoDetail1 != null) {
                            gameMap.put(gameInfo.gameID, gameInfoDetail1);
                        }
                    });
                } else {
                }
            }

            @Override
            public void onItemLongClick(RecyclerView recyclerView, View view, int position) {

            }
        });

        TextView startGameBtn = inputLayout.findViewById(R.id.start_game_btn);
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext(), android.R.style.Theme_Translucent_NoTitleBar);
        dialogBuilder.setView(inputLayout);
        AlertDialog dialog = dialogBuilder.create();
        startGameBtn.setOnClickListener(v -> {
            int selectIndex = gameListAdapter.getSelectPosition();
            if (selectIndex != -1) {
                ZegoGameInfoDetail gameInfoDetail = gameMap.get(viewModel.hostGameList.getValue().get(selectIndex).gameID);
                boolean isSoundCustomPlay = playByNative.isChecked();
                audioPlayer = new AudioPlayer();
                miniGameEngine.setGameContainer(gameLayout);
                Log.i(TAG, "[startGame][gameInfoDetail]: " + gameInfoDetail);
                currentGameID = gameInfoDetail.gameID;
                // 设置房间附加消息，指定游戏和房间ID

                RoomGameInfo roomGameInfo = new RoomGameInfo();
                roomGameInfo.gameID = gameInfoDetail.gameID;
                roomGameInfo.roomID = viewModel.roomID;
                roomGameInfo.gameMode = ZegoGameMode.ZegoGameModeHostsGame.value();
                roomGameInfo.isSoundCustomPlay = isSoundCustomPlay;
                // 主持人创建游戏房间后，把游戏信息设置到RTC的房间附加消息里
                rtcManager.setRoomExtraInfo(viewModel.roomID, ROOM_GAME_INFO, roomGameInfo.toString());
                loadGame(gameInfoDetail.gameID, ZegoGameMode.ZegoGameModeHostsGame, viewModel.roomID, isSoundCustomPlay);

                dialog.dismiss();
            } else {
                showToast(getString(R.string.please_chose_a_game));
            }
        });

        Window window = dialog.getWindow();
        window.setWindowAnimations(R.style.AnimBottom);
        window.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        window.setGravity(Gravity.BOTTOM);
        dialog.show();
    }

    private void onClickInGameMatchPlayBtn() {
        ViewGroup inputLayout = (ViewGroup) LayoutInflater.from(requireContext()).inflate(R.layout.select_game_dialog, null);
        RecyclerView gameRecyclerView = inputLayout.findViewById(R.id.game_list_recyclerview);
        RadioButton playByNative = inputLayout.findViewById(R.id.play_by_native);
        GridLayoutManager layoutManager = new GridLayoutManager(requireContext(), 4); // 每行4个
        gameRecyclerView.setLayoutManager(layoutManager);
        ChoseGameListAdapter gameListAdapter = new ChoseGameListAdapter();
        gameListAdapter.setSelectable(true);
        gameListAdapter.setDataList(viewModel.inGameMatchGameList.getValue());
        gameRecyclerView.setAdapter(gameListAdapter);
        gameListAdapter.setItemClickListener(new OnRecyclerViewItemClickListener() {
            @Override
            public void onItemClick(RecyclerView recyclerView, View view, int position) {
                ZegoGameInfo gameInfo = gameListAdapter.getDataList().get(position);
                ZegoGameInfoDetail gameInfoDetail = gameMap.get(gameInfo.gameID);
                if (gameInfoDetail == null) {
                    miniGameEngine.getGameInfo(gameInfo.gameID, (errorCode, gameInfoDetail1) -> {
                        if (errorCode != 0) {
                            showToast(String.format(getString(R.string.get_game_info_failed), ZegoConstants.getErrorMsg(errorCode)));
                        }
                        if (gameInfoDetail1 != null) {
                            gameMap.put(gameInfo.gameID, gameInfoDetail1);
                        }
                    });
                } else {
                }
            }

            @Override
            public void onItemLongClick(RecyclerView recyclerView, View view, int position) {

            }
        });

        TextView startGameBtn = inputLayout.findViewById(R.id.start_game_btn);
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext(), android.R.style.Theme_Translucent_NoTitleBar);
        dialogBuilder.setView(inputLayout);
        AlertDialog dialog = dialogBuilder.create();
        startGameBtn.setOnClickListener(v -> {
            int selectIndex = gameListAdapter.getSelectPosition();
            if (selectIndex != -1) {
                ZegoGameInfoDetail gameInfoDetail = gameMap.get(viewModel.inGameMatchGameList.getValue().get(selectIndex).gameID);
                boolean isSoundCustomPlay = playByNative.isChecked();
                audioPlayer = new AudioPlayer();
                miniGameEngine.setGameContainer(gameLayout);
                Log.i(TAG, "[startGame][gameInfoDetail]: " + gameInfoDetail);
                currentGameID = gameInfoDetail.gameID;
                // 设置房间附加消息，指定游戏和房间ID

                RoomGameInfo roomGameInfo = new RoomGameInfo();
                roomGameInfo.gameID = gameInfoDetail.gameID;
                roomGameInfo.roomID = viewModel.roomID;
                roomGameInfo.gameMode = ZegoGameMode.ZegoGameModeMatchInGame.value();
                roomGameInfo.isSoundCustomPlay = isSoundCustomPlay;
                // 主持人创建游戏房间后，把游戏信息设置到RTC的房间附加消息里
                rtcManager.setRoomExtraInfo(viewModel.roomID, ROOM_GAME_INFO, roomGameInfo.toString());
                loadGame(gameInfoDetail.gameID, ZegoGameMode.ZegoGameModeMatchInGame, viewModel.roomID, isSoundCustomPlay);

                dialog.dismiss();
            } else {
                showToast(getString(R.string.please_chose_a_game));
            }
        });

        Window window = dialog.getWindow();
        window.setWindowAnimations(R.style.AnimBottom);
        window.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        window.setGravity(Gravity.BOTTOM);
        dialog.show();
    }

    private void onClickPlayShowGameBtn() {
        ViewGroup inputLayout = (ViewGroup) LayoutInflater.from(requireContext()).inflate(R.layout.select_show_game_dialog, null);
        RecyclerView gameRecyclerView = inputLayout.findViewById(R.id.game_list_recyclerview);
        RadioButton playByNative = inputLayout.findViewById(R.id.play_by_native);
        GridLayoutManager layoutManager = new GridLayoutManager(requireContext(), 4); // 每行4个
        gameRecyclerView.setLayoutManager(layoutManager);
        ChoseGameListAdapter gameListAdapter = new ChoseGameListAdapter();
        gameListAdapter.setSelectable(true);
        gameListAdapter.setDataList(viewModel.showGameList.getValue());
        gameRecyclerView.setAdapter(gameListAdapter);
        gameListAdapter.setItemClickListener(new OnRecyclerViewItemClickListener() {
            @Override
            public void onItemClick(RecyclerView recyclerView, View view, int position) {
            }

            @Override
            public void onItemLongClick(RecyclerView recyclerView, View view, int position) {
            }
        });

        TextView startGameBtn = inputLayout.findViewById(R.id.start_game_btn);
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext(), android.R.style.Theme_Translucent_NoTitleBar);
        dialogBuilder.setView(inputLayout);
        AlertDialog dialog = dialogBuilder.create();
        startGameBtn.setOnClickListener(v -> {
            int selectIndex = gameListAdapter.getSelectPosition();
            if (selectIndex != -1) {
                boolean isSoundCustomPlay = playByNative.isChecked();
                audioPlayer = new AudioPlayer();
                miniGameEngine.setGameContainer(gameLayout);
                currentGameID = viewModel.showGameList.getValue().get(selectIndex).gameID;
                Log.i(TAG, "[startGame][currentGameID]: " + currentGameID);
                loadGame(currentGameID, ZegoGameMode.ZegoGameModeShowGame, viewModel.roomID, isSoundCustomPlay);

                dialog.dismiss();
            } else {
                showToast(getString(R.string.please_chose_a_game));
            }
        });

        Window window = dialog.getWindow();
        window.setWindowAnimations(R.style.AnimBottom);
        window.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        window.setGravity(Gravity.BOTTOM);
        dialog.show();
    }

    private void initMenu() {
        ArrayAdapter<Object> gameLanguageAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.gameLanguages);
        gameLanguageAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        miniGameEngine.setGameLanguage(ZegoConstants.gameLanguages[viewModel.gameLanguageSetting].gameLanguage);
        addCommandWithSelector(getString(R.string.game_language), gameLanguageAdapter, viewModel.gameLanguageSetting, value -> {
            ZegoConstants.GameLanguage gameLanguage = (ZegoConstants.GameLanguage)value;
            miniGameEngine.setGameLanguage(gameLanguage.gameLanguage);
        });

        addCommand(getString(R.string.get_music_switch), value -> {
            int ret = miniGameEngine.getGameMusicSwitch((errorCode, retValue) -> {
                if (errorCode == 0) {
                    showToast(getString(R.string.get_music_switch) + ":" + retValue);
                } else {
                    showToast(ZegoConstants.getErrorMsg(errorCode));
                }
            });
            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });

        addCommand(getString(R.string.set_music_switch), value -> {
            int ret = miniGameEngine.muteGameMusic(!isMusicOpen);
            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            } else {
                isMusicOpen = !isMusicOpen;
            }
        });

        addCommand(getString(R.string.get_music_volume), value -> {
            int ret = miniGameEngine.getGameMusicVolume((errorCode, retValue) -> {
                if (errorCode == 0) {
                    showToast(getString(R.string.get_music_volume) + "：" + retValue);
                } else {
                    showToast(ZegoConstants.getErrorMsg(errorCode));
                }
            });
            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });

        addCommandWithInput(getString(R.string.set_music_volume), "50", value -> {
            int ret = miniGameEngine.setGameMusicVolume(Integer.valueOf(value));
            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });

        addCommand(getString(R.string.get_vibrate_switch), value -> {
            int ret = miniGameEngine.getGameVibrateSwitch((errorCode, retValue) -> {
                if (errorCode == 0) {
                    showToast(getString(R.string.get_vibrate_switch) + ": " + retValue);
                } else {
                    showToast(ZegoConstants.getErrorMsg(errorCode));
                }
            });
            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });

        addCommand(getString(R.string.set_vibrate_switch), value -> {
            int ret = miniGameEngine.enableGameVibrate(!isVibrateOpen);
            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            } else {
                isVibrateOpen = !isVibrateOpen;
            }
        });

        addCommand(getString(R.string.get_sound_effect_switch), value -> {
            int ret = miniGameEngine.getGameSoundEffectSwitch((errorCode, retValue) -> {
                if (errorCode == 0) {
                    showToast(getString(R.string.get_sound_effect_switch) + ": " + retValue);
                } else {
                    showToast(ZegoConstants.getErrorMsg(errorCode));
                }
            });
            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });

        addCommand(getString(R.string.set_sound_effect_switch), value -> {
            int ret = miniGameEngine.muteGameSoundEffect(!isSoundEffectOpen);
            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            } else {
                isSoundEffectOpen = !isSoundEffectOpen;
            }
        });

        addCommand(getString(R.string.get_sound_effect_volume), value -> {
            int ret = miniGameEngine.getGameSoundEffectVolume((errorCode, retValue) -> {
                if (errorCode == 0) {
                    showToast(getString(R.string.get_sound_effect_volume) + ":" + retValue);
                } else {
                    showToast(ZegoConstants.getErrorMsg(errorCode));
                }
            });
            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });

        addCommandWithInput(getString(R.string.set_sound_effect_volume), "50", value -> {
            int ret = miniGameEngine.setGameSoundEffectVolume(Integer.valueOf(value));
            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });

        addCommand(getString(R.string.get_game_components), value -> {
            int ret = miniGameEngine.getGameComponentList((errorCode, componentList) -> {
                if (errorCode == 0) {
                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext());
                    dialogBuilder.setTitle(R.string.get_game_components).setMessage(gson.toJson(componentList)).setPositiveButton(R.string.confirm, (dialog, which) -> {
                    });
                    dialogBuilder.show();
                } else {
                    showToast(ZegoConstants.getErrorMsg(errorCode));
                }
            });

            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });

        addCommandWithInput(getString(R.string.show_game_components), "gameSetup", value -> {
            int ret = miniGameEngine.setGameComponent(value, true, (errorCode, isSuccess) -> {
                showToast(isSuccess ? "Success" : "Fail");
            });

            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });
        addCommandWithInput(getString(R.string.hide_game_components), "gameSetup", value -> {
            int ret = miniGameEngine.setGameComponent(value, false, (errorCode, isSuccess) -> {
                showToast(isSuccess ? "Success" : "Fail");
            });

            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });
        addCommand(getString(R.string.get_game_functions), value -> {
            int ret = miniGameEngine.getGameFunctionList((errorCode, functionList) -> {
                if (errorCode == 0) {
                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext());
                    dialogBuilder.setTitle(R.string.get_game_functions).setMessage(gson.toJson(functionList)).setPositiveButton(R.string.confirm, (dialog, which) -> {
                    });
                    dialogBuilder.show();
                } else {
                    showToast(ZegoConstants.getErrorMsg(errorCode));
                }
            });

            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });

        addCommandWithInput(getString(R.string.run_game_function), "setting", value -> {
            int ret = miniGameEngine.runGameFunction(value, (errorCode, isSuccess) -> {
                showToast(isSuccess ? "Success" : "Fail");
            });

            if (ret != 0) {
                showToast(ZegoConstants.getErrorMsg(ret));
            }
        });
        addCommandWithInput(getString(R.string.set_screen_height_width_rate), "1.10", value -> {
            ConstraintLayout.LayoutParams layoutParams = (ConstraintLayout.LayoutParams)gameLayout.getLayoutParams();
            layoutParams.height = (int)(gameLayout.getWidth() * Float.parseFloat(value));
            gameLayout.setLayoutParams(layoutParams);
        });
    }

    protected void onHostIDUpdate(String hostID) {

    }

    /**
     * 在游戏开始时把业务积分兑换成游戏积分，保证除了游戏中，其他都是自己在维护业务积分
     */
    protected void onGameStart() {
        loadingTv.setVisibility(View.VISIBLE);
        if (!TextUtils.isEmpty(currentGameID)) {
            DemoGameInfo demoGameInfo = viewModel.getDemoGameInfo(currentGameID);
            if (demoGameInfo != null) {
                final long tempPoint = demoGameInfo.gamePoint;
                demoGameInfo.gamePoint = 0;
                // 兑换游戏积分
            }
        } else {
            showToast("onGameStart：" + getString(R.string.game_id_is_empty));
        }
    }

    /**
     * 在游戏退出时把游戏积分兑换成业务积分，保证除了游戏中，其他都是自己在维护业务积分
     */
    protected void onGameQuit() {
        String gameID = currentGameID;
        if (!TextUtils.isEmpty(gameID)) {
            DemoGameInfo demoGameInfo = viewModel.getDemoGameInfo(gameID);
            if (demoGameInfo != null) {
                // 兑换游戏积分
            }
        }
    }

    // IZegoGameEngineHandler
    @Override
    public void onGameStateUpdate(ZegoGameState gameState) {
//        showToast("onGameStateUpdate:" + gameState);
    }

    @Override
    public void onGameStateUpdate(ZegoGameState gameState, int reasonCode) {
        showToast("onGameStateUpdate2:" + gameState + ", reasonCode:" + reasonCode);
    }

    @Override
    public void onGameLoadStateUpdate(ZegoGameLoadState gameLoadState) {
        showToast("onGameLoadStateUpdate:" + gameLoadState);
        if (gameLoadState == ZegoGameLoadState.ZegoGameLoading || gameLoadState == ZegoGameLoadState.ZegoGameLoaded) {
            loadingTv.setVisibility(View.GONE);
        }

//        if (gameLoadState == ZegoGameLoadState.ZegoGameLoaded) {
//            miniGameEngine.runGameFunction("autoJoin", (errorCode, result) -> {
//                showToast("autoJoin:" + result);
//            });
//        }
    }

    @Override
    public void onTokenWillExpire() {
        showToast("onTokenWillExpire");
    }

    @Override
    public void onPlayerStateUpdate(ZegoGamePlayerState playerEvent) {
        showToast(String.format(getString(R.string.update_player_status), playerEvent.toString()));
        isPlayer = playerEvent == ZegoGamePlayerState.ZegoGamePlayerStartPlay;
    }

    @Override
    public void onUnloaded(String gameID) {
        onGameQuit();
        currentGameID = "";
        isPlayingGame = false;
        isPlayer = false;
        updateButtonVisibility();
        if (audioPlayer != null) {
            audioPlayer.release();
            audioPlayer = null;
        };
    }

    @Override
    public void onPointsInsufficient(String gameID) {
        // 弹出补充积分对话框
    }

    @Override
    public void onGameSoundPlay(String name, boolean isPlay, String url, boolean isLoop, int volume) {
        if (audioPlayer != null) {
            audioPlayer.playSound(name, isPlay, url, isLoop, volume);
        }
        Log.i(TAG, "[onGameSoundPlay] name: " + name + ", isPlay:" + isPlay + ", isLoop:" + isLoop + ", volume:" + volume + ", url:" + url);
    }

    @Override
    public void onGameSoundVolumeChange(String name, int volume) {
        if (audioPlayer != null) {
            audioPlayer.changeSoundVolume(name, volume);
        }
        Log.i(TAG, "[onGameSoundVolumeChange] name: " + name + ", volume:" + volume);
    }

    @Override
    public void onGameError(int errorCode, String errorMessage) {
        Log.i(TAG, "[onGameError] errorCode: " + errorCode);
        showToast(String.format(getString(R.string.game_running_error), ZegoConstants.getErrorMsg(errorCode) + errorMessage));
    }

    @Override
    public void onActionEventUpdate(int actionID, String data) {
        Log.i(TAG, "[onActionEventUpdate] actionID: " + actionID + ", data:" + data);
        showToast("onActionEventUpdate - actionID:" + actionID + ", data:" + data);
    }

    @Override
    public void onGameResult(String result) {
        Log.i(TAG, "[onGameRoundResult] result: " + result);
        showToast("onGameRoundResult - result:" + result);
    }

    @Override
    public ZegoGameRobotConfig onRobotConfigRequire(String gameID) {
        Log.i(TAG, "[onRobotConfigRequire] gameID: " + gameID);
        String userName = ZegoConstants.getRandomName();
        String userAvatar = ZegoConstants.getRandomAvatar();
        return new ZegoGameRobotConfig(4, userName, userAvatar, 10, 13000, 20000);
    }

    @Override
    public void onMicStateChange(boolean isMute) {
        Log.i(TAG, "[onMicStateChange] mute: " + isMute);
        rtcManager.muteMicrophone(isMute);
    }

    @Override
    public void onSpeakerStateChange(List<String> userIDList, boolean isMute) {
        Log.i(TAG, "[onSpeakerStateChange] userIDList: " + userIDList.toArray() + ", isMute:" + isMute);
        for (String userID: userIDList) {
            if (!userID.equals(viewModel.userID)) {
                rtcManager.muteSpeaker(userID, isMute);
            }
        }
    }

    @Override
    public void onGameActionRequest(int actionID, String data, IZegoGameActionCallback callback) {
        // callback.onSuccess()是允许用户操作，如果客户有拦截需求，这里可以调用,
        // 调用太晚有可能导致失败，比如占位子，同意之前如果其他人已经坐下了，这里同意了也会占位失败。
        // callback.onFail(error) 来拦截，这样游戏中的操作就会不执行。error随意填就可以。
        // 具体的actionID对用游戏中的操作需要联系ZEGO技术支持。
//        showToast("[onGameActionRequest] actionID: " + actionID + ", data:" + data + "isAllow:" + isAllowRequest);
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext());
        dialogBuilder.setTitle(R.string.notify).setMessage("请求action：" + actionID + ", data:" + data).setPositiveButton(R.string.agree, (dialog, which) -> {
            callback.onSuccess();
        }).setNegativeButton(R.string.disagree,  (dialog, which) -> {
            callback.onFail(1);
        });
        dialogBuilder.setOnCancelListener(dialog -> requireActivity().onBackPressed());
        dialogBuilder.show();
    }
    // END -- IZegoGameEngineHandler

    // RTCEngineManager.IRTCEventListener interfaces
    @Override
    public void onStreamAdd(ArrayList<RTCEngineManager.StreamInfo> streamInfoList) {
    }

    @Override
    public void onStreamRemove(ArrayList<RTCEngineManager.StreamInfo> streamInfoList) {
    }

    @Override
    public void onRoomStateChanged(RTCEngineManager.RoomStateChangeReason reason) {
        if (reason == RTCEngineManager.RoomStateChangeReason.KickOut) {
            handler.post(() -> {
                AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext());
                dialogBuilder.setTitle(R.string.notify).setMessage(R.string.already_login_on_other_device).setPositiveButton(R.string.confirm, (dialog, which) -> {
                    requireActivity().onBackPressed();
                });
                dialogBuilder.setOnCancelListener(dialog -> requireActivity().onBackPressed());
                dialogBuilder.show();
            });
        } else if (reason == RTCEngineManager.RoomStateChangeReason.Disconnect || reason == RTCEngineManager.RoomStateChangeReason.LoginFailed) {
            handler.post(() -> {
                AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext());
                dialogBuilder.setTitle(R.string.notify).setMessage(R.string.you_are_offline).setPositiveButton(R.string.confirm, (dialog, which) -> {
                    requireActivity().onBackPressed();
                });
                dialogBuilder.setOnCancelListener(dialog -> requireActivity().onBackPressed());
                dialogBuilder.show();
            });
        }
    }

    @Override
    public void onRoomExtraInfoUpdate(String roomID, ArrayList<ZegoRoomExtraInfo> arrayList) {
        ArrayList<ZegoRoomExtraInfo> roomExtraInfoList = rtcManager.getRoomExtraInfoList();
        if (roomExtraInfoList != null && roomExtraInfoList.size() > 0) {
            // 解析游戏的id和房间id
            for (ZegoRoomExtraInfo roomExtraInfo : roomExtraInfoList) {
                hostID = roomExtraInfo.updateUser.userID;
                onHostIDUpdate(hostID);
                if (roomExtraInfo.key.equals(ROOM_GAME_INFO)) {
                    String value = roomExtraInfo.value;
                    if (!TextUtils.isEmpty(value)) {
                        roomGameInfo = gson.fromJson(value, RoomGameInfo.class);
                        if (roomGameInfo == null) {
                            showToast(getString(R.string.game_room_info_error));
                            return;
                        }

                        if (isPlayingGame) {
                            // 先判断是否已经在玩游戏了
                            ViewGroup inputLayout = (ViewGroup) LayoutInflater.from(requireContext()).inflate(R.layout.dialog_join_host_game, null);
                            AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireContext(), android.R.style.Theme_Translucent_NoTitleBar);

                            TextView agree = inputLayout.findViewById(R.id.agree_btn);
                            TextView deny = inputLayout.findViewById(R.id.deny_btn);

                            dialogBuilder.setView(inputLayout);
                            AlertDialog dialog = dialogBuilder.create();
                            agree.setOnClickListener(v -> {
                                dialog.dismiss();
                                miniGameEngine.unloadGame(true);
                                miniGameEngine.setGameContainer(gameLayout);
                                currentGameID = roomGameInfo.gameID;
                                audioPlayer = new AudioPlayer();
                                // 加载游戏
                                loadGame(currentGameID, ZegoGameMode.fromValue(roomGameInfo.gameMode), roomGameInfo.roomID, roomGameInfo.isSoundCustomPlay);
                            });

                            deny.setOnClickListener(v -> {
                                dialog.dismiss();
                            });
                            Window window = dialog.getWindow();
                            window.setGravity(Gravity.CENTER);
                            dialog.show();
                        } else {
                            miniGameEngine.setGameContainer(gameLayout);
                            currentGameID = roomGameInfo.gameID;
                            audioPlayer = new AudioPlayer();
                            // 加载游戏
                            loadGame(currentGameID, ZegoGameMode.fromValue(roomGameInfo.gameMode), roomGameInfo.roomID, roomGameInfo.isSoundCustomPlay);
                        }
                    } else {
                        // 不处理，等待游戏的unload通知
                        if (roomGameInfo != null) {
                            miniGameEngine.unloadGame(true);
                        }
                        updateButtonVisibility();
                    }
                }
            }
        }
    }

    @Override
    public void onRoomOnlineUserCountUpdate(String roomID, int count) {
        memberCountTv.setText(String.format(getString(R.string.online_member), count));
    }

    @Override
    public void onRoomUserUpdate(List<RTCEngineManager.RTCUser> newUserList) {
    }

    @Override
    public void onPublisherStreamEvent(ZegoStreamEvent eventID, String streamID, String extraInfo) {
    }
    // END -- RTCEngineManager.IRTCEventListener interfaces

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        rtcManager.removeRTCEventListener(this);
        viewModel.refreshGamePoint.removeObserver(refreshPointObserver);
        if (viewModel.isHost) {
            rtcManager.stopPublishStream();
            if (!TextUtils.isEmpty(currentGameID)) {
                miniGameEngine.unloadGame(true);
                // 这里需执行onGameQuit，不能等onUnloadGame回调
                onGameQuit();
                rtcManager.setRoomExtraInfo(viewModel.roomID, ROOM_GAME_INFO, "");
                updateButtonVisibility();
            }
        } else {
            if (!TextUtils.isEmpty(currentGameID)) {
                miniGameEngine.unloadGame(true);
                // 这里需执行onGameQuit，不能等onUnloadGame回调
                onGameQuit();
                updateButtonVisibility();
            }
        }

        if (audioPlayer != null) {
            audioPlayer.release();
            audioPlayer = null;
        }
        rtcManager.logoutRoom();
    }

    private Observer<Integer> refreshPointObserver = new Observer<Integer>() {
        @Override
        public void onChanged(Integer update) {
            if (update == 0) {
                return;
            }
            // 当上个游戏退出后，快速进新的游戏。有可能兑出积分的操作执行很慢导致这时才收到刷新通知。
            if (!TextUtils.isEmpty(currentGameID)) {
                DemoGameInfo demoGameInfo = viewModel.getDemoGameInfo(currentGameID);
                if (demoGameInfo != null && demoGameInfo.gamePoint > 0) {
                    final long tempPoint = demoGameInfo.gamePoint;
                    demoGameInfo.gamePoint = 0;
                    // 兑换游戏积分
                }
            }
        }
    };

    protected void updateGameParams(ZegoGameInfoDetail gameInfoDetail, RadioGroup gameSpinRadioGroup, RadioGroup playerNumRadioGroup) {
        gameSpinRadioGroup.removeAllViews();
        ColorStateList csl = getResources().getColorStateList(R.color.zego_radio_btn_icon_color);
        int textColor = getResources().getColor(R.color.white);
        if (gameInfoDetail.pointList != null && gameInfoDetail.pointList.size() > 0) {
            boolean firstChecked = false;
            for (Long point : gameInfoDetail.pointList) {
                MaterialRadioButton radioButton = new MaterialRadioButton(requireContext());
                radioButton.setText(point + "");
                radioButton.setTextSize(14);
                radioButton.setTextColor(textColor);
                radioButton.setButtonTintList(csl);
                gameSpinRadioGroup.addView(radioButton);
                radioButton.setOnCheckedChangeListener((buttonView, isChecked) -> {
                    if (isChecked) {
                        maxPlayerNum = Integer.valueOf(buttonView.getText().toString());
                    }
                });
                if (!firstChecked) {
                    firstChecked = true;
                    radioButton.setChecked(true);
                }
            }
        }

        playerNumRadioGroup.removeAllViews();
        if (gameInfoDetail.playerNum != null && gameInfoDetail.playerNum.size() > 0) {
            boolean firstChecked = false;
            for (Integer playerNum : gameInfoDetail.playerNum) {
                MaterialRadioButton radioButton = new MaterialRadioButton(requireContext());
                radioButton.setText(playerNum + "");
                radioButton.setTextSize(14);
                radioButton.setTextColor(textColor);
                radioButton.setButtonTintList(csl);
                playerNumRadioGroup.addView(radioButton);

                if (!firstChecked) {
                    firstChecked = true;
                    maxPlayerNum = playerNum;
                    radioButton.setChecked(true);
                }
            }
        }
    }

    protected void updateButtonVisibility() {
    }

    protected void loadGame(String gameID, ZegoGameMode zegoGameMode, String roomID, boolean isCustomPlaySound) {
        ZegoGameInfoDetail gameInfoDetail = gameMap.get(currentGameID);
        if (gameInfoDetail == null) {
            miniGameEngine.getGameInfo(currentGameID, ((errorCode, object) -> {
                if (object != null) {
                    gameMap.put(currentGameID, object);
                    _loadGame(gameID, zegoGameMode, roomID, isCustomPlaySound, object);
                }
            }));
        } else {
            _loadGame(gameID, zegoGameMode, roomID, isCustomPlaySound, gameInfoDetail);
        }
    }

    private void _loadGame(String gameID, ZegoGameMode zegoGameMode, String roomID, boolean isCustomPlaySound, ZegoGameInfoDetail gameInfoDetail) {
        isPlayer = false;
        HashMap<String, Object> gameConfig = new HashMap<>();
        gameConfig.put(ZegoMiniGameEngine.GAME_CONFIG_ROOM_ID, roomID);
        gameConfig.put(ZegoMiniGameEngine.GAME_CONFIG_CUSTOM_PLAY_SOUND, isCustomPlaySound);
        gameConfig.put(ZegoMiniGameEngine.POINT_ICON_URL, "https://bobilive-com-test.jieyou.shop/h5web/GameH5/showDemo/game_bean.png");
        gameConfig.put(ZegoMiniGameEngine.USE_ROBOT, true);
        long point = 1000;
        if (gameInfoDetail != null && gameInfoDetail.pointList != null && gameInfoDetail.pointList.size() > 0) {
            point = gameInfoDetail.pointList.get(0);
        }
        gameConfig.put(ZegoMiniGameEngine.MIN_GAME_POINT, 0);
        HashMap<String, Object> specificConfig = new HashMap<>();
        specificConfig.put("paddingTop", "10");  // 单位dp
        specificConfig.put("paddingBottom", "10");
        gameConfig.put("specificConfig", specificConfig);
        miniGameEngine.loadGame(gameID,
                zegoGameMode,
                gameConfig, (errorCode2, object2) -> {
                    if (ZegoGameEngineError.SUCCESS == errorCode2) {
                        isPlayingGame = true;
                        onGameStart();
                        updateButtonVisibility();
                    } else {
                        showToast(String.format(getString(R.string.load_game_error), ZegoConstants.getErrorMsg(errorCode2)));
                    }
                });
    }

    @Override
    public void onPause() {
        super.onPause();
        if (audioPlayer != null) {
            audioPlayer.pause();
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        if (audioPlayer != null) {
            audioPlayer.resume();
        }
    }

    protected void showToast(String msg) {
        CommonUtils.showToast(msg);
    }

    // 处理添加功能菜单 BEGIN
    interface IRunFunc {
        void onRunFunc(String value);
    }

    interface IRunFunc2 {
        void onRunFunc(Object value);
    }

    private void addCommand(String content, IRunFunc runFunc) {
        addCommand(menuLayout, content, runFunc);
    }

    private void addCommandWithInput(String content, IRunFunc runFunc) {
        addCommandWithInput(menuLayout, content, runFunc);
    }

    private void addCommandWithInput(String content, String defaultText, IRunFunc runFunc) {
        addCommandWithInput(menuLayout, content, defaultText, runFunc);
    }

    private void addCommand(LinearLayout layout, String content, IRunFunc runFunc) {
        Button button = new Button(new ContextThemeWrapper(requireContext(), R.style.testButtonStyle));
        button.setText(content);
        button.setOnClickListener(v -> {
            runFunc.onRunFunc("");
        });
        layout.addView(button);
    }

    private void addCommandWithInput(LinearLayout layout, String content, IRunFunc runFunc) {
        Button button = new Button(new ContextThemeWrapper(requireContext(), R.style.testButtonStyle));
        button.setText(content);
        EditText editText = new EditText(new ContextThemeWrapper(requireContext(), R.style.testButtonStyle));
        editText.setBackground(getResources().getDrawable(R.drawable.edittext_bg));
        LinearLayout commandLayout = new LinearLayout(requireContext());
        commandLayout.setOrientation(LinearLayout.HORIZONTAL);
        commandLayout.addView(editText, new ViewGroup.LayoutParams(CommonUtils.dp2px(requireContext(), 100f), CommonUtils.dp2px(requireContext(), 40f)));
        commandLayout.addView(button, new ViewGroup.LayoutParams(CommonUtils.dp2px(requireContext(), 190f), CommonUtils.dp2px(requireContext(), 40f)));
        button.setOnClickListener(v -> {
            runFunc.onRunFunc(editText.getText().toString());
        });
        commandLayout.setPadding(10, 10, 10, 10);
        layout.addView(commandLayout);
    }

    private void addCommandWithInput(LinearLayout layout, String content, String defaultText, IRunFunc runFunc) {
        Button button = new Button(new ContextThemeWrapper(requireContext(), R.style.testButtonStyle));
        button.setText(content);
        EditText editText = new EditText(new ContextThemeWrapper(requireContext(), R.style.testButtonStyle));
        editText.setText(defaultText);
        editText.setBackground(getResources().getDrawable(R.drawable.edittext_bg));
        LinearLayout commandLayout = new LinearLayout(requireContext());
        commandLayout.setOrientation(LinearLayout.HORIZONTAL);
        commandLayout.addView(editText, new ViewGroup.LayoutParams(CommonUtils.dp2px(requireContext(), 100f), CommonUtils.dp2px(requireContext(), 40f)));
        commandLayout.addView(button, new ViewGroup.LayoutParams(CommonUtils.dp2px(requireContext(), 190f), CommonUtils.dp2px(requireContext(), 40f)));
        button.setOnClickListener(v -> {
            runFunc.onRunFunc(editText.getText().toString());
        });
        commandLayout.setPadding(10, 10, 10, 10);
        layout.addView(commandLayout);
    }

    private void addCommandWithSelector(String content, ArrayAdapter<Object> adapter, int selectIndex, IRunFunc2 runFunc) {
        addCommandWithSelector(menuLayout, content, adapter, selectIndex, runFunc);
    }

    private void addCommandWithSelector(LinearLayout layout, String content, ArrayAdapter<Object> adapter, int selectIndex, IRunFunc2 runFunc) {
        TextView text = new TextView(layout.getContext());
        text.setText(content);
        Spinner spinner = new Spinner(layout.getContext());
        spinner.setAdapter(adapter);
        spinner.setSelection(selectIndex);
        spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                Object selectItem = adapter.getItem(position);
                runFunc.onRunFunc(selectItem);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
        LinearLayout commandLayout = new LinearLayout(requireContext());
        commandLayout.setBackgroundColor(Color.parseColor("#ffffff"));
        commandLayout.setOrientation(LinearLayout.HORIZONTAL);
        commandLayout.addView(text, new ViewGroup.LayoutParams(CommonUtils.dp2px(requireContext(), 100f), CommonUtils.dp2px(requireContext(), 40f)));
        commandLayout.addView(spinner, new ViewGroup.LayoutParams(CommonUtils.dp2px(requireContext(), 190f), CommonUtils.dp2px(requireContext(), 40f)));
        commandLayout.setPadding(10, 10, 10, 10);
        layout.addView(commandLayout);
    }
    // 处理添加功能菜单 END
}