package im.zego.minigameenginedemo.ui.main;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;

import im.zego.minigameenginedemo.R;
import im.zego.minigameenginedemo.model.LiveModel;

public class LiveListAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {
    private ArrayList<LiveModel> dataList;
    private OnRecyclerViewItemClickListener itemClickListener;
    private RecyclerView recyclerView;

    @Override
    public void onAttachedToRecyclerView(@NonNull RecyclerView recyclerView) {
        super.onAttachedToRecyclerView(recyclerView);
        this.recyclerView = recyclerView;
    }

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.live_item_layout, parent, false);
        GameViewHolder gameViewHolder = new GameViewHolder(view);
        return gameViewHolder;
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        GameViewHolder gameViewHolder = (GameViewHolder)holder;
        LiveModel liveModel = dataList.get(position);
        Context context = recyclerView.getContext();
        gameViewHolder.roomTypeTv.setText(liveModel.roomType == LiveModel.RoomType.LIVE_ROOM ?
                context.getString(R.string.live_room) : liveModel.roomType == LiveModel.RoomType.CHAT_ROOM ?
                context.getString(R.string.chat_room) : context.getString(R.string.game_room));
        gameViewHolder.itemTv.setText(liveModel.liveName);
        gameViewHolder.itemView.setOnClickListener(v -> {
            if (itemClickListener != null) {
                itemClickListener.onItemClick(recyclerView, v, position);
            }
        });

        gameViewHolder.itemView.setOnLongClickListener(v -> {
            if (itemClickListener != null) {
                itemClickListener.onItemClick(recyclerView, v, position);
            }
            return false;
        });
    }

    @Override
    public int getItemCount() {
        return dataList != null ? dataList.size() : 0;
    }

    public void setItemClickListener(OnRecyclerViewItemClickListener itemClickListener) {
        this.itemClickListener = itemClickListener;
    }

    public static class GameViewHolder extends RecyclerView.ViewHolder {
        public TextView roomTypeTv;
        public TextView itemTv;
        public GameViewHolder(@NonNull View itemView) {
            super(itemView);
            roomTypeTv = itemView.findViewById(R.id.item_tv);
            itemTv = itemView.findViewById(R.id.item_num_tv);
        }
    }

    public void setDataList(ArrayList<LiveModel> dataList) {
        this.dataList = dataList;
        notifyDataSetChanged();
    }

}
