package im.zego.minigameenginedemo.ui.main;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import im.zego.minigameenginedemo.R;
import im.zego.minigameenginedemo.RTCEngineManager;
import im.zego.minigameenginedemo.ZegoConstants;
import im.zego.minigameenginedemo.model.LiveModel;
import im.zego.minigameenginedemo.view.SpaceItemDecoration;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link LiveListFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class LiveListFragment extends Fragment {
    protected MiniGameMainViewModel viewModel;
    protected RecyclerView liveListRecyclerView;
    RTCEngineManager rtcEngineManager = RTCEngineManager.getInstance();

    protected LiveListAdapter liveListAdapter = new LiveListAdapter();

    public LiveListFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @return A new instance of fragment LiveListFragment.
     */
    public static LiveListFragment newInstance() {
        return new LiveListFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        viewModel = new ViewModelProvider(requireActivity()).get(MiniGameMainViewModel.class);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View view = inflater.inflate(getInflateLayoutRes(), container, false);
        liveListRecyclerView = view.findViewById(R.id.recyclerview_live_list);
        GridLayoutManager layoutManager = new GridLayoutManager(requireContext(), 3); // 每行2个
        liveListRecyclerView.setLayoutManager(layoutManager);
        liveListRecyclerView.addItemDecoration(new SpaceItemDecoration(22));
        liveListAdapter.setDataList(ZegoConstants.liveModelList);
        liveListRecyclerView.setAdapter(liveListAdapter);
        liveListAdapter.setItemClickListener(new OnRecyclerViewItemClickListener() {
            @Override
            public void onItemClick(RecyclerView recyclerView, View view, int position) {
                onItemViewClick(position);
            }

            @Override
            public void onItemLongClick(RecyclerView recyclerView, View view, int position) {

            }
        });
        return view;
    }

    protected int getInflateLayoutRes() {
        return R.layout.fragment_live_list;
    }

    protected void onItemViewClick(int position) {
        viewModel.isHost = false;
        loginRoom(position);
    }

    protected void loginRoom(int position) {
        String newRoomID = ZegoConstants.getRoomID(position);
        if (!rtcEngineManager.isInRoom()) {
            rtcEngineManager.logoutRoom();
        }

        rtcEngineManager.loginRoom(newRoomID, newRoomID, viewModel.userID, viewModel.userName, (errorCode, result) -> {
            viewModel.roomID = newRoomID;
            viewModel.roomType = ZegoConstants.liveModelList.get(position).roomType;
            Fragment fragment = viewModel.roomType == LiveModel.RoomType.LIVE_ROOM ? LiveFragment.newInstance() :
                    viewModel.roomType == LiveModel.RoomType.CHAT_ROOM ?
                    ChatFragment.newInstance() : GameRoomFragment.newInstance();
            requireActivity().getSupportFragmentManager().beginTransaction()
                    .add(R.id.container, fragment, "LiveFragment")
                    .addToBackStack(null)
                    .commit();
        });
    }

    @Override
    public String toString() {
        return getString(R.string.live_mall);
    }
}