package im.zego.minigameenginedemo.ui.main;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.lifecycle.ViewModelProvider;
import androidx.viewpager.widget.ViewPager;

import com.bumptech.glide.Glide;
import com.bumptech.glide.request.RequestOptions;

import java.util.ArrayList;

import im.zego.minigameenginedemo.KeyCenter;
import im.zego.minigameenginedemo.R;
import im.zego.minigameenginedemo.view.ZegoCustomButton;

public class MiniGameMainFragment extends Fragment {

    private MiniGameMainViewModel viewModel;
    private ArrayList<Fragment> fragmentList = new ArrayList<>();
    private ZegoCustomButton gameMallBtn;
    private ZegoCustomButton liveRoomBtn;
    private ZegoCustomButton startLiveBtn;
    private ZegoCustomButton mineBtn;
    private ZegoCustomButton cloudGameBtn;
    public static MiniGameMainFragment newInstance() {
        return new MiniGameMainFragment();
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        viewModel = new ViewModelProvider(requireActivity()).get(MiniGameMainViewModel.class);
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container,
                             @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_mini_game_main, container, false);
        TextView envTv = view.findViewById(R.id.env_tv);
        envTv.setText("(appID: " + KeyCenter.appID + ")\nuserID: " + viewModel.userID);
        TextView userName = view.findViewById(R.id.user_name);
        ImageView userAvatar = view.findViewById(R.id.user_avatar);
        ImageView setupImageView = view.findViewById(R.id.setup);
        RequestOptions options = new RequestOptions().circleCrop();
        Glide.with(requireContext()).load(viewModel.userAvatar).apply(options).into(userAvatar);
        userName.setText(viewModel.userName);
        ViewPager tabViewPager = view.findViewById(R.id.tab_viewpager);
        fragmentList.add(GameListFragment.newInstance());
        fragmentList.add(LiveListFragment.newInstance());
        fragmentList.add(StartLiveListFragment.newInstance());
        fragmentList.add(CloudGameListFragment.newInstance());
        fragmentList.add(MyInfoFragment.newInstance());
        // 保持每个page不回收
        tabViewPager.setOffscreenPageLimit(fragmentList.size());

        tabViewPager.addOnPageChangeListener(new ViewPager.SimpleOnPageChangeListener() {
            @Override
            public void onPageSelected(int position) {
                if (position == 0) {
                    gameMallBtn.setSelected(true);
                    liveRoomBtn.setSelected(false);
                    startLiveBtn.setSelected(false);
                    cloudGameBtn.setSelected(false);
                    mineBtn.setSelected(false);
                } else if (position == 1) {
                    gameMallBtn.setSelected(false);
                    liveRoomBtn.setSelected(true);
                    startLiveBtn.setSelected(false);
                    cloudGameBtn.setSelected(false);
                    mineBtn.setSelected(false);
                } else if (position == 2) {
                    gameMallBtn.setSelected(false);
                    liveRoomBtn.setSelected(false);
                    startLiveBtn.setSelected(true);
                    cloudGameBtn.setSelected(false);
                    mineBtn.setSelected(false);
                } else if (position == 3) {
                    gameMallBtn.setSelected(false);
                    liveRoomBtn.setSelected(false);
                    startLiveBtn.setSelected(false);
                    cloudGameBtn.setSelected(true);
                    mineBtn.setSelected(false);
                } else {
                    gameMallBtn.setSelected(false);
                    liveRoomBtn.setSelected(false);
                    startLiveBtn.setSelected(false);
                    cloudGameBtn.setSelected(false);
                    mineBtn.setSelected(true);
                }
            }
        });

        tabViewPager.setAdapter(new FragmentPagerAdapter(requireActivity().getSupportFragmentManager()) {
            @Override
            public int getCount() {
                return fragmentList.size();
            }

            @Override
            public Fragment getItem(int position) {
                return fragmentList.get(position);
            }

            @Override
            public CharSequence getPageTitle(int position) {
                return getItem(position).toString();
            }
        });

        gameMallBtn = view.findViewById(R.id.game_mall_btn);
        gameMallBtn.setText(getString(R.string.game_mall));
        gameMallBtn.setImageResource(R.drawable.game_mall_btn);
        gameMallBtn.setSelected(true);
        gameMallBtn.setOnClickListener(v -> {
            tabViewPager.setCurrentItem(0, true);
        });

        liveRoomBtn = view.findViewById(R.id.live_mall_btn);
        liveRoomBtn.setText(getString(R.string.live_mall));
        liveRoomBtn.setImageResource(R.drawable.live_mall_btn);
        liveRoomBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                tabViewPager.setCurrentItem(1, true);
            }
        });

        startLiveBtn = view.findViewById(R.id.live_btn);
        startLiveBtn.setText(getString(R.string.start_live));
        startLiveBtn.setImageResource(R.drawable.start_live_btn);
        startLiveBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                tabViewPager.setCurrentItem(2, true);
            }
        });

        cloudGameBtn = view.findViewById(R.id.cloud_game_btn);
        cloudGameBtn.setText(getString(R.string.cloud_game));
        cloudGameBtn.setImageResource(R.drawable.cloud_game_btn);
        cloudGameBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                tabViewPager.setCurrentItem(3, true);
            }
        });

        mineBtn = view.findViewById(R.id.my_info_btn);
        mineBtn.setText(getString(R.string.my_info));
        mineBtn.setImageResource(R.drawable.mine_btn);
        mineBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                tabViewPager.setCurrentItem(4, true);
            }
        });

        setupImageView.setOnClickListener(v -> {
            Fragment settingFragment = SettingFragment.newInstance();
            requireActivity().getSupportFragmentManager().beginTransaction()
                    .add(R.id.container, settingFragment, "SettingFragment")
                    .addToBackStack(null)
                    .commit();
        });
        return view;
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        fragmentList.clear();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
    }

}