package im.zego.minigameenginedemo.ui.main;

import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.ViewModel;

import java.util.List;

import im.zego.minigameengine.ZegoGameInfo;
import im.zego.minigameengine.ZegoGameLanguage;
import im.zego.minigameenginedemo.ZegoConstants;
import im.zego.minigameenginedemo.model.DemoGameInfo;
import im.zego.minigameenginedemo.model.LiveModel;

public class MiniGameMainViewModel extends ViewModel {
    public String userName; // 用户名
    public String userID; // 用户ID
    public String roomID; // 房间ID
    public LiveModel.RoomType roomType; // 房间类型
    public String userAvatar; // 用户头像
    public int languageSetting = 0; // App语言设置，默认为0
    public int gameLanguageSetting = 0; // 游戏语言设置，默认为0
    public ZegoGameLanguage systemLanguage = ZegoGameLanguage.zh_CN; // 系统语言，默认为中文
    public boolean isHost = false; // 是否为主播
    public MutableLiveData<Long> businessPoint = new MutableLiveData<>(ZegoConstants.BUSINESS_POINT_RESET); // 业务积分值
    public MutableLiveData<List<ZegoGameInfo>> hostGameList = new MutableLiveData<>(); // 主播带玩游戏列表
    public MutableLiveData<List<ZegoGameInfo>> showGameList = new MutableLiveData<>(); // 秀场游戏列表
    public MutableLiveData<List<ZegoGameInfo>> fullScreenGameList = new MutableLiveData<>(); // 全屏游戏列表
    public MutableLiveData<List<ZegoGameInfo>> cloudGameList = new MutableLiveData<>(); // 云渲染游戏列表
    public MutableLiveData<List<ZegoGameInfo>> inGameMatchGameList = new MutableLiveData<>(); // 游戏内匹配游戏列表
    public MutableLiveData<List<DemoGameInfo>> allGameList = new MutableLiveData<>(); // 所有的游戏列表
    public MutableLiveData<Integer> refreshGamePoint = new MutableLiveData<>(0); // 通知我的信息界面刷新游戏积分信息

    /**
     * 根据游戏ID获取对应的 DemoGameInfo 对象
     * @param gameID 游戏ID
     * @return 对应的 DemoGameInfo 对象，如果找不到返回null
     */
    public DemoGameInfo getDemoGameInfo(String gameID) {
        List<DemoGameInfo> demoGameInfoList = allGameList.getValue();
        for (DemoGameInfo demoGameInfo: demoGameInfoList) {
            if (demoGameInfo.gameInfo.gameID.equals(gameID)) {
                return demoGameInfo;
            }
        }

        return null;
    }
}