package im.zego.minigameenginedemo.ui.main;

import static im.zego.minigameenginedemo.utils.CommonUtils.showToast;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import im.zego.minigameengine.ZegoMiniGameEngine;
import im.zego.minigameenginedemo.R;
import im.zego.minigameenginedemo.ZegoConstants;
import im.zego.minigameenginedemo.model.DemoGameInfo;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link MyInfoFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class MyInfoFragment extends Fragment {
    private MiniGameMainViewModel viewModel; // ViewModel对象，用于管理数据
    private TextView businessPointTv; // 用于显示业务积分数量的TextView

    public MyInfoFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @return A new instance of fragment MyInfoFragment.
     */
    public static MyInfoFragment newInstance() {
        return new MyInfoFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        viewModel = new ViewModelProvider(requireActivity()).get(MiniGameMainViewModel.class); // 初始化ViewModel
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_my_info, container, false);
        businessPointTv = view.findViewById(R.id.business_point_count_tv); // 获取业务积分数量的TextView
        businessPointTv.setText(viewModel.businessPoint.getValue() + ""); // 设置业务积分数量显示

        // 设置业务积分数量的按钮点击事件
        view.findViewById(R.id.reset_business_point_count_btn).setOnClickListener(v -> {
            viewModel.businessPoint.setValue(ZegoConstants.BUSINESS_POINT_RESET); // 重置积分数量
        });

        // 观察业务积分数量的变化，当业务积分数量发生变化时更新业务积分数量的显示
        viewModel.businessPoint.observe(requireActivity(), newValue -> {
            businessPointTv.setText(newValue + "");
        });

        RecyclerView gameListRecyclerView = view.findViewById(R.id.game_list_recyclerview); // 获取游戏列表的RecyclerView
        LinearLayoutManager layoutManager = new LinearLayoutManager(requireContext()); // 创建线性布局管理器
        layoutManager.setOrientation(LinearLayoutManager.VERTICAL); // 设置布局方向为垂直方向
        gameListRecyclerView.setLayoutManager(layoutManager); // 设置RecyclerView的布局管理器
        DemoGameListAdapter gameListAdapter = new DemoGameListAdapter(); // 创建游戏列表适配器
        gameListRecyclerView.setAdapter(gameListAdapter); // 设置RecyclerView的适配器

        // 设置游戏列表适配器中的积分补充按钮点击事件
        gameListAdapter.setButtonClickListener(new DemoGameListAdapter.IButtonClickListener() {
            @Override
            public void onLongClickIcon(int position, DemoGameListAdapter.GameViewHolder viewHolder, DemoGameInfo info) {
                ZegoMiniGameEngine.getInstance().clearGameResources(info.gameInfo.gameID);
                showToast(getString(R.string.clear_game_resource)); // 显示业务积分不足的提示信息
            }

            @Override
            public void onClickExchangeButton(int position, DemoGameListAdapter.GameViewHolder viewHolder, DemoGameInfo info) {
                if (viewModel.businessPoint.getValue() >= ZegoConstants.EXCHANGE_POINT_NUMBER) {
                    info.gamePoint += ZegoConstants.EXCHANGE_POINT_NUMBER; // 增加游戏积分数量
                    viewModel.businessPoint.setValue(viewModel.businessPoint.getValue() - ZegoConstants.EXCHANGE_POINT_NUMBER); // 减少业务积分数量
                    gameListAdapter.notifyItemChanged(position); // 通知适配器刷新指定位置的数据
                } else {
                    showToast(getString(R.string.business_point_is_not_enough)); // 显示业务积分不足的提示信息
                }
            }

            @Override
            public void onClickPreloadButton(int position, DemoGameListAdapter.GameViewHolder viewHolder, DemoGameInfo info) {
                if (info.isDownloading) {
                    ZegoMiniGameEngine.getInstance().unPreloadGame(info.gameInfo.gameID);
                    info.downloadProgress = 0;
                    info.isDownloading = false;
                    gameListAdapter.notifyItemChanged(position);
                } else {
                    info.isDownloading = true;
                    ZegoMiniGameEngine.getInstance().preloadGame(info.gameInfo.gameID, (errorCode, progress) -> {
                        info.downloadProgress = progress;
                        if (progress == 100) {
                            info.isDownloading = false;
                        }
                        if (errorCode != 0) {
                            info.downloadProgress = 0;
                            showToast(String.format(getString(R.string.game_running_error), ZegoConstants.getErrorMsg(errorCode)));
                        }
                        gameListAdapter.notifyItemChanged(position);
                    });
                }
            }
        });

        // 观察所有游戏列表的变化，当游戏列表发生变化时更新游戏列表适配器的数据
        viewModel.allGameList.observe(requireActivity(), gameList -> {
            gameListAdapter.setDataList(gameList);
        });

        // 观察刷新游戏积分的标志位，当标志位发生变化时刷新游戏列表适配器
        viewModel.refreshGamePoint.observe(requireActivity(), update -> {
            gameListAdapter.notifyDataSetChanged();
        });

        return view; // 返回填充后的视图
    }

    @Override
    public String toString() {
        return getString(R.string.my_info); // 返回该Fragment的描述信息
    }
}