package im.zego.minigameenginedemo.ui.main;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import im.zego.minigameengine.IZegoGameActionCallback;
import im.zego.minigameengine.IZegoGameEngineHandler;
import im.zego.minigameengine.ZegoGameComponent;
import im.zego.minigameengine.ZegoGameEngineError;
import im.zego.minigameengine.ZegoGameLoadState;
import im.zego.minigameengine.ZegoGameMode;
import im.zego.minigameengine.ZegoGamePlayerState;
import im.zego.minigameengine.ZegoGameRobotConfig;
import im.zego.minigameengine.ZegoGameState;
import im.zego.minigameengine.ZegoMiniGameEngine;
import im.zego.minigameenginedemo.KeyCenter;
import im.zego.minigameenginedemo.R;
import im.zego.minigameenginedemo.ZegoConstants;
import im.zego.minigameenginedemo.model.DemoGameInfo;
import im.zego.minigameenginedemo.utils.CommonUtils;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link PlayGameFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class PlayGameFragment extends Fragment {
    private static final String TAG = "PlayGameFragment";

    public static final String PARAM_GAME_ID = "GAME_ID";
    private static final String ARG_PARAM2 = "param2";

    private String gameID;
    private String mParam2;

    private TextView closeGameBtn;
    private ViewGroup gameContainer;
    private TextView loadingTv;
    private boolean isPlayer = false;

    private ZegoMiniGameEngine miniGameEngine = ZegoMiniGameEngine.getInstance();
    private MiniGameMainViewModel viewModel;
    private Handler handler = new Handler(Looper.getMainLooper());

    public PlayGameFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @param gameID game ID.
     * @param param2 Parameter 2.
     * @return A new instance of fragment PlayGameFragment.
     */
    public static PlayGameFragment newInstance(String gameID, String param2) {
        PlayGameFragment fragment = new PlayGameFragment();
        Bundle args = new Bundle();
        args.putString(PARAM_GAME_ID, gameID);
        args.putString(ARG_PARAM2, param2);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // 获取传递的参数
        if (getArguments() != null) {
            gameID = getArguments().getString(PARAM_GAME_ID);
            mParam2 = getArguments().getString(ARG_PARAM2);
        }
        // 创建ViewModel实例
        viewModel = new ViewModelProvider(requireActivity()).get(MiniGameMainViewModel.class);

        // 设置游戏引擎回调
        miniGameEngine.setGameEngineHandler(new IZegoGameEngineHandler() {
            @Override
            public void onTokenWillExpire() {
                showToast("onTokenWillExpire");
            }

            @Override
            public void onGameLoadStateUpdate(ZegoGameLoadState gameLoadState) {
                showToast("onGameLoadState:" + gameLoadState);
                // 根据游戏加载状态更新UI
                if (gameLoadState == ZegoGameLoadState.ZegoGameLoading || gameLoadState == ZegoGameLoadState.ZegoGameLoaded) {
                    loadingTv.setVisibility(View.GONE);
                }
            }

            @Override
            public void onGameStateUpdate(ZegoGameState gameState) {
//                showToast("onGameState:" + gameState);
            }

            @Override
            public void onGameStateUpdate(ZegoGameState gameState, int reasonCode) {
                showToast("onGameState:" + gameState + ", reasonCode:" + reasonCode);
            }

            @Override
            public void onPlayerStateUpdate(ZegoGamePlayerState playerEvent) {
                showToast(String.format(getString(R.string.update_player_status), playerEvent.toString()));
                // 根据玩家状态更新isPlayer变量
                isPlayer = playerEvent == ZegoGamePlayerState.ZegoGamePlayerStartPlay;
            }

            @Override
            public void onUnloaded(String gameID) {
                // 游戏卸载时执行的操作
                onGameQuit();
                PlayGameFragment.this.gameID = "";
                handler.post(() -> {
                    FragmentActivity fragmentActivity = getActivity();
                    if (fragmentActivity != null) {
                        fragmentActivity.onBackPressed();
                    }
                });

                isPlayer = false;
            }

            @Override
            public void onPointsInsufficient(String gameID) {
                // 弹出补充金币对话框
            }

            @Override
            public void onGameSoundPlay(String name, boolean isPlay, String url, boolean isLoop, int volume) {
                Log.i(TAG, "[onGameSoundPlay] name: " + name + ", isPlay:" + isPlay + ", isLoop:" + isLoop + ", volume:" + volume + ", url:" + url);
            }

            @Override
            public void onGameSoundVolumeChange(String name, int volume) {
                Log.i(TAG, "[onGameSoundVolumeChange] name: " + name + ", volume:" + volume);
            }

            @Override
            public void onGameError(int errorCode, String errorMessage) {
                Log.i(TAG, "[onGameError] errorCode: " + errorCode);
                showToast(String.format(getString(R.string.game_running_error), ZegoConstants.getErrorMsg(errorCode) + errorMessage));
            }

            @Override
            public void onActionEventUpdate(int actionID, String data) {
                Log.i(TAG, "[onActionEventUpdate] actionID: " + actionID + ", data:" + data);
                showToast("onActionEventUpdate - actionID:" + actionID + ", data:" + data);
            }

            @Override
            public void onGameResult(String result) {
                Log.i(TAG, "[onGameRoundResult] result: " + result);
                showToast("onGameRoundResult - result:" + result);
            }

            @Override
            public ZegoGameRobotConfig onRobotConfigRequire(String gameID) {
                Log.i(TAG, "[onRobotConfigRequire] gameID: " + gameID);
                // 生成随机的用户名和用户头像，并返回游戏机器人配置
                String userName = ZegoConstants.getRandomName();
                String userAvatar = ZegoConstants.getRandomAvatar();
                return new ZegoGameRobotConfig(4, userName, userAvatar, 10, 13000, 20000);
            }

            @Override
            public void onMicStateChange(boolean isMute) {

            }

            @Override
            public void onSpeakerStateChange(List<String> userIDList, boolean isMute) {

            }

            @Override
            public void onGameActionRequest(int actionID, String data, IZegoGameActionCallback callback) {
                // callback.onSuccess()是允许用户操作，如果客户有拦截需求，这里可以调用
                // callback.onFail(error) 来拦截，这样游戏中的操作就会不执行。 error随意填就可以。
                // 具体的actionID对用游戏中的操作需要联系ZEGO技术支持。
                callback.onSuccess();
            }

            @Override
            public void onComponentListUpdate(List<ZegoGameComponent> componentList) {
                for (ZegoGameComponent componentInfo : componentList) {
                    // componentInfo.name 表示可控制的组件名称
                    // 可继续调用 ZegoMiniGameEngine.getInstance().setGameComponent(String, boolean, IZegoCommonCallback) 设置组件的隐藏与否
                }
            }
        });
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View rootView = inflater.inflate(R.layout.fragment_play_game, container, false);
        gameContainer = rootView.findViewById(R.id.game_container);
        loadingTv = rootView.findViewById(R.id.loading_tv);
        closeGameBtn = rootView.findViewById(R.id.close_game_btn);
        closeGameBtn.setOnClickListener(v -> {
            miniGameEngine.unloadGame(true);
        });

        rootView.findViewById(R.id.close_room_btn).setOnClickListener(v -> {
            if (!TextUtils.isEmpty(gameID)) {
                miniGameEngine.unloadGame(true);
                // 这里需执行onGameQuit，不能等onUnloadGame回调
                onGameQuit();
                gameID = null;
            }
            requireActivity().onBackPressed();
        });

        miniGameEngine.setGameContainer(gameContainer);
        miniGameEngine.getGameInfo(gameID, ((errorCode, gameInfoDetail) -> {
            if (getActivity() == null) {
                Log.w(TAG, "getActivity() == null");
                return;
            }
            isPlayer = false;
            HashMap<String, Object> gameConfig = new HashMap<>();
            gameConfig.put(ZegoMiniGameEngine.GAME_CONFIG_ROOM_ID, viewModel.roomID);
            gameConfig.put(ZegoMiniGameEngine.GAME_CONFIG_CUSTOM_PLAY_SOUND, false);
            gameConfig.put(ZegoMiniGameEngine.USE_ROBOT, true);
            long point = 1000;
            if (gameInfoDetail != null && gameInfoDetail.pointList != null && gameInfoDetail.pointList.size() > 0) {
                point = gameInfoDetail.pointList.get(0);
            }
            gameConfig.put(ZegoMiniGameEngine.MIN_GAME_POINT, 0);
            HashMap<String, Object> specificConfig = new HashMap<>();
            specificConfig.put("paddingTop", "20");  // 单位dp
            specificConfig.put("paddingBottom", "60");
            gameConfig.put("specificConfig", specificConfig);
            miniGameEngine.setGameLanguage(ZegoConstants.gameLanguages[viewModel.gameLanguageSetting].gameLanguage);
            miniGameEngine.loadGame(gameID,
                    ZegoGameMode.ZegoGameModeMallGame,
                    gameConfig, (errorCode2, object) -> {
                        if (getActivity() == null) {
                            return;
                        }
                        if (ZegoGameEngineError.SUCCESS == errorCode2) {
                            onGameStart();
                        } else {
                            showToast(String.format(getString(R.string.load_game_error), ZegoConstants.getErrorMsg(errorCode2)));
                            handler.post(() -> {
                                requireActivity().onBackPressed();
                            });
                        }
                    });
            viewModel.refreshGamePoint.observe(requireActivity(), refreshPointObserver);
        }));

        return rootView;
    }

    @Override
    public void onDestroyView() {
        viewModel.refreshGamePoint.removeObserver(refreshPointObserver);
        if (!TextUtils.isEmpty(gameID)) {
            miniGameEngine.unloadGame(true);
            // 这里需执行onGameQuit，不能等onUnloadGame回调
            onGameQuit();
        }
        super.onDestroyView();
    }

    /**
     * 在游戏开始时把业务积分兑换成游戏积分，保证除了游戏中，其他都是自己在维护业务积分
     */
    private void onGameStart() {
        loadingTv.setVisibility(View.VISIBLE);
        if (!TextUtils.isEmpty(gameID)) {
            DemoGameInfo demoGameInfo = viewModel.getDemoGameInfo(gameID);
            Log.i(TAG, PlayGameFragment.this + "[onGameStart] demoGameInfo:" + demoGameInfo + "," + demoGameInfo.gamePoint);
            if (demoGameInfo != null && demoGameInfo.gamePoint > 0) {
                final long tempPoint = demoGameInfo.gamePoint;
                demoGameInfo.gamePoint = 0;
                // 兑换游戏积分
            }
        } else {
            Activity activity = getActivity();
            if (activity != null) {
                showToast("onGameStart：" + activity.getString(R.string.game_id_is_empty));
            }
        }
    }

    /**
     * 在游戏退出时把游戏积分兑换成业务积分，保证除了游戏中，其他都是自己在维护业务积分
     */
    private void onGameQuit() {
        if (!TextUtils.isEmpty(gameID)) {
            final String tempGameID = gameID;
            DemoGameInfo demoGameInfo = viewModel.getDemoGameInfo(tempGameID);
            if (demoGameInfo != null) {
                // 兑换游戏积分
            }
        }
    }

    private Observer<Integer> refreshPointObserver = new Observer<Integer>() {
        @Override
        public void onChanged(Integer update) {
            Log.i(TAG, PlayGameFragment.this + "[refreshPointObserver] onChanged update:" + update);
            if (update == 0) {
                return;
            }
            // 当上个游戏退出后，快速进新的游戏。有可能兑出积分的操作执行很慢导致这时才收到刷新通知。
            Log.i(TAG, PlayGameFragment.this + "[refreshPointObserver] onChanged");
            if (!TextUtils.isEmpty(gameID)) {
                DemoGameInfo demoGameInfo = viewModel.getDemoGameInfo(gameID);
                Log.i(TAG, PlayGameFragment.this + "[refreshPointObserver] demoGameInfo:" + demoGameInfo + "," + demoGameInfo.gamePoint);
                if (demoGameInfo != null && demoGameInfo.gamePoint > 0) {
                    final long tempPoint = demoGameInfo.gamePoint;
                    demoGameInfo.gamePoint = 0;
                    // 兑换游戏积分
                }
            }
        }
    };

    private void showToast(String msg) {
        CommonUtils.showToast(msg);
    }
}