package im.zego.minigameenginedemo.ui.main;

import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.fragment.app.Fragment;

import im.zego.minigameenginedemo.R;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link SelectFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class SelectFragment extends Fragment {

    // TODO: Rename parameter arguments, choose names that match
    // the fragment initialization parameters, e.g. ARG_ITEM_NUMBER
    private static final String ARG_TITLE = "title";
    private static final String ARG_BACK_STRING = "back_string";
    private static final String ARG_STRING_ARRAY = "string_array";
    private static final String ARG_SELECTED_INDEX = "selected_index";

    // TODO: Rename and change types of parameters
    private String title;
    private String backString;
    private String stringArray[];
    private int selectedIndex;
    private LinearLayout selectLayout;
    private SelectItemView selectedItem;
    private SelectFragmentCallback callback;
    public SelectFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @param title Fragment's title.
     * @param backString back button' text.
     * @param stringArray select string array.
     * @param selectIndex selected item index
     * @return A new instance of fragment SelectFragment.
     */
    public static SelectFragment newInstance(String title, String backString, String[] stringArray, int selectIndex, SelectFragmentCallback callback) {
        SelectFragment fragment = new SelectFragment();
        Bundle args = new Bundle();
        args.putString(ARG_TITLE, title);
        args.putString(ARG_BACK_STRING, backString);
        args.putStringArray(ARG_STRING_ARRAY, stringArray);
        args.putInt(ARG_SELECTED_INDEX, selectIndex);
        fragment.setArguments(args);
        fragment.setCallback(callback);
        return fragment;
    }

    private void setCallback(SelectFragmentCallback callback) {
        this.callback = callback;
    }


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (getArguments() != null) {
            title = getArguments().getString(ARG_TITLE);
            backString = getArguments().getString(ARG_BACK_STRING);
            stringArray = getArguments().getStringArray(ARG_STRING_ARRAY);
            selectedIndex = getArguments().getInt(ARG_SELECTED_INDEX);
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View rootView = inflater.inflate(R.layout.fragment_select, container, false);
        rootView.findViewById(R.id.back_btn).setOnClickListener(v -> {
            requireActivity().onBackPressed();
            if (callback != null) {
                callback.onResult((Integer)selectedItem.getTag());
                callback = null;
            }
        });
        selectLayout = rootView.findViewById(R.id.select_layout);
        TextView backTV = rootView.findViewById(R.id.back_btn);
        backTV.setText(backString);
        TextView titleTV = rootView.findViewById(R.id.title);
        titleTV.setText(title);
        for (int i = 0; i < stringArray.length; i++) {
            final SelectItemView itemView = new SelectItemView(requireContext());
            itemView.setTag(i);
            itemView.setText(stringArray[i]);
            if (i == selectedIndex) {
                selectedItem = itemView;
            }

            itemView.setSelected(i == selectedIndex);
            itemView.setBottomItem(i == stringArray.length - 1);
            selectLayout.addView(itemView, ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
            itemView.setOnClickListener(v -> {
                selectedItem.setSelected(false);
                selectedItem = itemView;
                selectedItem.setSelected(true);
            });
        }
        return rootView;
    }

    @Override
    public void onPause() {
        super.onPause();
        if (callback != null) {
            callback.onResult((Integer)selectedItem.getTag());
            callback = null;
        }
    }

    interface SelectFragmentCallback {
        void onResult(int selectIndex);
    }
}