package im.zego.minigameenginedemo.ui.main;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;
import java.util.List;

import im.zego.minigameenginedemo.R;
import im.zego.minigameenginedemo.RTCEngineManager;

public class SelectMemberListAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder>{
    private List<RTCEngineManager.RTCUser> dataList;
    private int selectBit[];
    private int maxSelectCount = 0;
    private int selectCount = 0;

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.select_member_item_layout, parent, false);
        SelectMemberViewHolder selectMemberViewHolder = new SelectMemberViewHolder(view);
        return selectMemberViewHolder;
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        SelectMemberViewHolder selectMemberViewHolder = (SelectMemberViewHolder)holder;
        RTCEngineManager.RTCUser user = dataList.get(position);
        selectMemberViewHolder.memberNameTv.setText(user.userName);
        selectMemberViewHolder.checkBox.setChecked(selectBit[position] == 1);
        selectMemberViewHolder.checkBox.setOnCheckedChangeListener((buttonView, isChecked) -> {
            if (isChecked) {
                if (selectCount < maxSelectCount) {
                    selectCount += 1;
                } else {
                    buttonView.setChecked(false);
                    return;
                }
            } else {
                selectCount -= 1;
            }
            selectBit[position] = isChecked ? 1 : 0;
        });
    }

    @Override
    public int getItemCount() {
        return dataList == null ? 0 : dataList.size();
    }

    public void setMaxSelectCount(int maxSelectCount) {
        this.maxSelectCount = maxSelectCount;
    }

    public static class SelectMemberViewHolder extends RecyclerView.ViewHolder {
        public TextView memberNameTv;
        public CheckBox checkBox;
        public SelectMemberViewHolder(@NonNull View itemView) {
            super(itemView);
            memberNameTv = itemView.findViewById(R.id.member_name);
            checkBox = itemView.findViewById(R.id.check_box);
        }
    }

    public void setDataList(List<RTCEngineManager.RTCUser> dataList) {
        this.dataList = dataList;
        selectBit = new int[dataList == null ? 0 : dataList.size()];
        notifyDataSetChanged();
    }

    public void reset() {
        for (int i = 0; i < selectBit.length; i++) {
            selectBit[i] = 0;
        }
        notifyDataSetChanged();
    }

    public List<RTCEngineManager.RTCUser> getSelectUserList() {
        List<RTCEngineManager.RTCUser> selectList = new ArrayList<>();
        for (int i = 0; i < selectBit.length; i++) {
            if (selectBit[i] == 1) {
                selectList.add(dataList.get(i));
            }
        }

        return selectList;
    }
}
