package im.zego.minigameenginedemo.ui.main;

import static im.zego.minigameenginedemo.MiniGameMainActivity.SP_FILENAME;
import static im.zego.minigameenginedemo.MiniGameMainActivity.SP_GAME_LANGUAGE;
import static im.zego.minigameenginedemo.MiniGameMainActivity.SP_LANGUAGE;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import im.zego.minigameengine.ZegoGameLanguage;
import im.zego.minigameenginedemo.R;
import im.zego.minigameenginedemo.ZegoConstants;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link SettingFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class SettingFragment extends Fragment {

    private MiniGameMainViewModel viewModel;

    public SettingFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @return A new instance of fragment SettingFragment.
     */
    public static SettingFragment newInstance() {
        SettingFragment fragment = new SettingFragment();
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        viewModel = new ViewModelProvider(requireActivity()).get(MiniGameMainViewModel.class);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View rootView = inflater.inflate(R.layout.fragment_setting, container, false);
        rootView.findViewById(R.id.back_btn).setOnClickListener(v -> {
            requireActivity().onBackPressed();
        });

        TextView languageValue = rootView.findViewById(R.id.language_value);
        String languages[] = new String[] {getString(R.string.language_system), getString(R.string.language_zh), getString(R.string.language_en)};
        languageValue.setText(languages[viewModel.languageSetting]);
        rootView.findViewById(R.id.language_setup_layout).setOnClickListener(v -> {
            // 进入设置语言
            Fragment selectFragment = SelectFragment.newInstance("", getString(R.string.common_return), languages, viewModel.languageSetting, selectIndex -> {
                if (viewModel.languageSetting != selectIndex) {
                    viewModel.languageSetting = selectIndex;
                    SharedPreferences sp = requireContext().getSharedPreferences(SP_FILENAME, Context.MODE_PRIVATE);
                    sp.edit().putInt(SP_LANGUAGE, selectIndex).commit();
                    final Intent intent = requireContext().getPackageManager().getLaunchIntentForPackage(requireContext().getPackageName());
                    intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                    startActivity(intent);
                    System.exit(0);
                }
            });
            requireActivity().getSupportFragmentManager().beginTransaction()
                    .add(R.id.container, selectFragment, "SelectFragment")
                    .addToBackStack(null)
                    .commit();
        });


        TextView gameLanguageValue = rootView.findViewById(R.id.game_language_value);
        String gameLanguages[] = new String[ZegoConstants.gameLanguages.length];
        for (int i = 0; i < ZegoConstants.gameLanguages.length; i++) {
            gameLanguages[i] = ZegoConstants.gameLanguages[i].text;
        }
        gameLanguageValue.setText(gameLanguages[viewModel.gameLanguageSetting]);
        rootView.findViewById(R.id.game_language_setup_layout).setOnClickListener(v -> {
            // 进入设置语言
            Fragment selectFragment = SelectFragment.newInstance("", getString(R.string.common_return), gameLanguages, viewModel.gameLanguageSetting, selectIndex -> {
                if (viewModel.gameLanguageSetting != selectIndex) {
                    viewModel.gameLanguageSetting = selectIndex;
                    gameLanguageValue.setText(gameLanguages[viewModel.gameLanguageSetting]);
                    SharedPreferences sp = requireContext().getSharedPreferences(SP_FILENAME, Context.MODE_PRIVATE);
                    sp.edit().putInt(SP_GAME_LANGUAGE, selectIndex).commit();
                }
            });
            requireActivity().getSupportFragmentManager().beginTransaction()
                    .add(R.id.container, selectFragment, "SelectFragment")
                    .addToBackStack(null)
                    .commit();
        });
        return rootView;
    }

}