package im.zego.minigameenginedemo.utils;

import android.app.Application;
import android.content.Context;
import android.os.Environment;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.widget.Toast;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import im.zego.minigameenginedemo.IExecuteCallback;
import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

public class CommonUtils {
    public static int dp2px(Context context, float dpValue) {
        float scale = context.getResources().getDisplayMetrics().density;
        return (int)(dpValue * scale + 0.5f);
    }

    public static File getDataBaseFile(Context context) {
        File result = null;
        if (context == null){
            result = Environment.getDataDirectory();
        } else {
            result = context.getExternalFilesDir(null);
            if (result == null) {
                result = context.getExternalCacheDir();
            }
            if (result == null) {
                result = context.getFilesDir();
            }
            if (result == null){
                result = context.getCacheDir();
            }
        }

        return result;
    }

    public static void downloadFile(String url, String savePath, IExecuteCallback<String> callback){
        final long startTime = System.currentTimeMillis();
        Log.i("DOWNLOAD","startTime="+startTime);
        File path = new File(savePath);
        if (!path.exists()) {
            path.mkdirs();
        }
        File file = new File(path, url.substring(url.lastIndexOf("/") + 1));
        if (file.exists()) {
            callback.onResult(0, file.getAbsolutePath());
            return;
        }

        OkHttpClient okHttpClient = new OkHttpClient();

        Request request = new Request.Builder().url(url).build();
        okHttpClient.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                // 下载失败
                e.printStackTrace();
                Log.i("DOWNLOAD","download failed");
                callback.onResult(-1, "");
            }
            @Override
            public void onResponse(Call call, Response response) throws IOException {
                InputStream is = null;
                byte[] buf = new byte[2048];
                int len = 0;
                FileOutputStream fos = null;
                // 储存下载文件的目录
                try {
                    is = response.body().byteStream();
                    long total = response.body().contentLength();
                    fos = new FileOutputStream(file);
                    long sum = 0;
                    while ((len = is.read(buf)) != -1) {
                        fos.write(buf, 0, len);
                        sum += len;
                        int progress = (int) (sum * 1.0f / total * 100);
                        // 下载中
//                        listener.onDownloading(progress);
                    }
                    fos.flush();
                    // 下载完成
                    callback.onResult(0, file.getAbsolutePath());
                } catch (Exception e) {
                    e.printStackTrace();
                    callback.onResult(-2, "");
                    Log.i("DOWNLOAD","download failed");
                } finally {
                    try {
                        if (is != null)
                            is.close();
                    } catch (IOException e) {
                    }
                    try {
                        if (fos != null)
                            fos.close();
                    } catch (IOException e) {
                    }
                }
            }
        });
    }

    public static Application getApplication() {
        try {
            Application application = (Application) Class.forName("android.app.ActivityThread").getMethod("currentApplication").invoke(null, (Object[]) null);
            return application;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static void showToast(String msg) {
        new Handler(Looper.getMainLooper()).post(() -> {
            Toast.makeText(getApplication(), msg, Toast.LENGTH_SHORT).show();
        });
    }
}
